"
 Copyright (C) 2014 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBParseTreeLintRule subclass:#RBAssignmentInConditionRule
	instanceVariableNames:'variablesPossiblyLeftUninitialized'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Experimental Rules'
!

!RBAssignmentInConditionRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) 2014 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
!

documentation
"
    UNFINIHSED
    the idea is that if you have an assignment only along one branch of a conditional,
    the value of the variable is harder to understand after the branch (as you have to make sure, that it is not
    left uninitialized.
    However, currently, we get many false positives, especially from lazy getters.
    Should further investigate and improve; possibly not complaining when the if is a nil-check.
"
! !

!RBAssignmentInConditionRule class methodsFor:'accessing'!

tagsBasic
    "Return an array of tags (symbols) of given rule. Any rule may
     arbitrary number of tags. Tag values are not predefined/fixed.
     See class documentation for list of common tags"

    ^#( broken )

    "Created: / 10-02-2017 / 17:56:24 / cg"
! !

!RBAssignmentInConditionRule methodsFor:'accessing'!

group
        ^ 'Possible Bugs'
!

name
        ^ 'Assignment inside condition'
!

rationale
    |vars|
    
    result isNil ifTrue:[^ self rationaleString].
    variablesPossiblyLeftUninitialized isEmptyOrNil ifTrue:[^ self rationaleString].
    
    vars := Set new.
    variablesPossiblyLeftUninitialized keysAndValuesDo:[:mthd :selectors |
        vars addAll:selectors.
    ].    
    ^ self rationaleWithVariables:vars

    "Modified: / 10-02-2017 / 17:52:22 / cg"
!

rationaleString
    ^ 
'Assignments should either be done in both branches of a condition
or outside a condition.
<P>
As these assignments may or may not be executed,
it may be harder to understand if and how a variable is set in the followup code.
<P>
This is an unfinished lint rule under construction (possibly with too many false positives)'

    "Created: / 10-02-2017 / 17:46:59 / cg"
!

rationaleWithAnchorForMethod:aMethod
    "called by the smallint browser, when a linted method is selected."

    ^ self rationaleWithVariables:(variablesPossiblyLeftUninitialized at:aMethod ifAbsent:#())

    "Created: / 10-02-2017 / 17:42:37 / cg"
    "Modified (comment): / 26-02-2017 / 12:33:39 / cg"
!

rationaleWithVariables:vars
    "called by the smallint browser, when a linted method is selected."

    ^ String streamContents:[:s |
        s nextPutAll:self rationaleString.
        vars notEmpty ifTrue:[
            s nextPutAll:'<p>Variables: ';
              nextPutAll:(vars asOrderedCollection sort asStringWith:', ')
        ].
    ].

    "Created: / 10-02-2017 / 17:47:33 / cg"
    "Modified (comment): / 26-02-2017 / 12:33:44 / cg"
! !

!RBAssignmentInConditionRule methodsFor:'initialization'!

initialize
        super initialize.
        self matcher 
                matchesAnyOf: #(
                        '`@expr ifTrue: [ `@.Statements1. `var := `@object. `@.Statements2. ]'

                        '`@expr ifFalse: [ `@.Statements1. `var := `@object. `@.Statements2. ]'

                        '`@expr ifTrue: [ `@.Statements1. `var := `@object. `@.Statements2. ]
                                ifFalse: [ `@.Statements3. `var2 := `@object2. `@.Statements4. ]'

                        '`@expr ifFalse: [ `@.Statements1. `var := `@object. `@.Statements2. ]
                                ifTrue: [ `@.Statements3. `var2 := `@object2. `@.Statements4. ]'

                        '`@expr ifNil: [ `@.Statements1. `var := `@object. `@.Statements2. ]'

                        '`@expr ifNotNil: [ `@.Statements1. `var := `@object. `@.Statements2. ]'

                        '`@expr ifNil: [ `@.Statements1. `var := `@object. `@.Statements2. ]
                                ifNotNil: [ `@.Statements3. `var2 := `@object2. `@.Statements4. ]'

                        '`@expr ifNotNil: [ `@.Statements1. `var := `@object. `@.Statements2. ]
                                ifNil: [ `@.Statements3. `var2 := `@object2. `@.Statements4. ]'
                 )
                do: [ :node :answer :owner :context |
                    |rslt 
                     varsUsedAfter varsSetBefore var1UsedAfter var1SetBefore var2UsedAfter var2SetBefore 
                     varNode varName varNode2 varName2 containingSequence idx
                     ifMessages isGuarded codeAfterReached
                     var1InScope var2InScope varsInScope|

                    rslt := answer.    
                    answer isNil ifTrue:[
                        varNode := context at:(RBPatternVariableNode named:'`var').
                        varName := varNode name.

                        (varNode2 := context at:(RBPatternVariableNode named:'`var2') ifAbsent:nil) notNil ifTrue:[
                            varName2 := varNode2 name.
                        ].    
                        (varName2 isNil or:[varName2 ~= varName]) ifTrue:[
                            isGuarded := false.
                            ifMessages := #( 'ifTrue:' 'ifFalse:' 'ifTrue:ifFalse:' 'ifFalse:ifTrue:'
                                             'ifNil:' 'ifNotNil:' 'ifNil:ifNotNil:' 'ifNotNil:ifNil:' ).

                            ( #( 'ifTrue:' 'ifFalse:' ) includes:node selector) ifTrue:[
                                node receiver isMessage ifTrue:[
                                    ( #( 'isNil' ) includes:node receiver selector) ifTrue:[
                                        node receiver receiver = varNode ifTrue:[
                                            isGuarded := true.
                                        ] ifFalse:[    
                                            "/ self halt.
                                        ]
                                    ] ifFalse:[
                                        ( #( '==' '~~' ) includes:node receiver selector) ifTrue:[
                                            ((node receiver receiver = varNode)
                                              or:[ node receiver arguments first = varNode ]
                                            ) ifTrue:[
                                                isGuarded := true.
                                            ] ifFalse:[    
                                                "/ self halt.
                                            ].
                                        ] ifFalse:[
                                            "/ self halt.
                                        ].
                                    ].
                                ].
                            ].
                            ( #( 'ifNil:' 'ifNotNil:' ) includes:node selector) ifTrue:[
                                node receiver = varNode ifTrue:[
                                    isGuarded := true.
                                ] ifFalse:[
                                    self halt.
                                ].
                            ].
                            (node arguments conform:[:eachBlock | eachBlock body statements last isReturn]) ifTrue:[
                            ] ifFalse:[    

                                isGuarded ifFalse:[
                                    containingSequence := node detectParentForWhich:[:p | p isSequence].

                                    varsUsedAfter := varsSetBefore := false.
                                    var1UsedAfter := var2UsedAfter := var1SetBefore := var2SetBefore := false.
                                    var1InScope := var2InScope := varsInScope := true.
                                    codeAfterReached := true.

                                    [ varsInScope 
                                        and:[varsUsedAfter not 
                                        and:[varsSetBefore not 
                                        and:[containingSequence notNil]]] 
                                    ] whileTrue:[
                                        idx := containingSequence statements identityIndexOf:node.

                                        "/ check if var is defined before
                                        (var1SetBefore and:[varName2 isNil or:[var2SetBefore]]) ifFalse:[
                                            containingSequence statements from:1 to:idx-1 do:[:eachPrefixStatement |
                                                var1InScope ifTrue:[
                                                    var1SetBefore ifFalse:[
                                                        (eachPrefixStatement assigns: varName ) ifTrue:[ var1SetBefore := true ].
                                                    ].    
                                                ].    
                                                varName2 notNil ifTrue:[
                                                    var2InScope ifTrue:[
                                                        var2SetBefore ifFalse:[
                                                            (eachPrefixStatement assigns: varName2 ) ifTrue:[ var2SetBefore := true ].
                                                        ].
                                                    ].
                                                ].
                                            ].
                                        ].    
                                        varsSetBefore := var1SetBefore and:[ varName2 isNil or:[var2SetBefore]].

                                        codeAfterReached ifTrue:[
                                            "/ check if var is used in statements following
                                            containingSequence statements from:idx+1 do:[:eachFollowupStatement |
                                                var1InScope ifTrue:[
                                                    var1UsedAfter ifFalse:[
                                                        (eachFollowupStatement references: varName ) ifTrue:[ var1UsedAfter := true ].
                                                    ].
                                                ].
                                                varName2 notNil ifTrue:[
                                                    var2InScope ifTrue:[
                                                        var2UsedAfter ifFalse:[
                                                            (eachFollowupStatement references: varName2 ) ifTrue:[ var2UsedAfter := true ].
                                                        ].
                                                    ].
                                                ].
                                            ].
                                            varsUsedAfter := var1UsedAfter and:[ varName2 isNil or:[var2UsedAfter]].
                                            containingSequence statements last isReturn ifTrue:[
                                                codeAfterReached := false.
                                            ].    
                                        ] ifFalse:[
                                            var1UsedAfter := var2UsedAfter := varsUsedAfter := false.
                                        ].    

                                        (containingSequence temporaries contains:[:tmp | tmp name = varName]) ifTrue:[
                                            var1InScope := false. "/ to break the loop when both are defined
                                        ].
                                        varName2 notNil ifTrue:[
                                            (containingSequence temporaries contains:[:tmp | tmp name = varName2]) ifTrue:[
                                                var2InScope := false. "/ to break the loop when both are defined
                                            ].
                                        ].
                                        varsInScope := var1InScope or:[ varName2 notNil and:[var2InScope]].
                                        containingSequence := containingSequence detectParentForWhich:[:p | p isSequence].
                                    ].
                                    
                                    (containingSequence isNil and:[varsInScope]) ifTrue:[
                                        "/ an inst- or other var.
                                        "/ ignore.
                                        var1InScope ifTrue:[var1SetBefore := true].
                                        var2InScope ifTrue:[var2SetBefore := varName2 isNil or:[true]].
                                        varsSetBefore := var1SetBefore and:[var2SetBefore].
                                    ].
                                    
                                    (varsSetBefore not and:[varsUsedAfter]) ifTrue:[
                                        |mthd|

                                        mthd := matcher currentSearchContext compiledMethod.
                                        variablesPossiblyLeftUninitialized isNil ifTrue:[
                                            variablesPossiblyLeftUninitialized := Dictionary new.
                                        ].
                                        (var1SetBefore and:[var1UsedAfter]) ifTrue:[
                                            (variablesPossiblyLeftUninitialized at:mthd ifAbsentPut:[Set new]) add:varName.
                                            result
                                                addSearchAction:(RBEnvironmentStringSelectAction 
                                                                selectFrom:varNode start to:varNode stop 
                                                                inMethod:mthd).
                                            rslt := node
                                        ].
                                        varName2 notNil ifTrue:[
                                            (var2SetBefore and:[var2UsedAfter]) ifTrue:[
                                                (variablesPossiblyLeftUninitialized at:mthd ifAbsentPut:[Set new]) add:varName2.
                                                result
                                                    addSearchAction:(RBEnvironmentStringSelectAction 
                                                                    selectFrom:varNode2 start to:varNode2 stop 
                                                                    inMethod:mthd).
                                                rslt := node
                                            ].
                                        ].    
                                    ].
                                ].
                            ]
                        ].
                    ].
                    rslt.
                ]

    "Modified: / 10-02-2017 / 17:53:38 / cg"
    "Modified (format): / 28-11-2017 / 17:01:30 / cg"
! !

!RBAssignmentInConditionRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

