"
 Copyright (C) 2017 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBParseTreeLintRule subclass:#RBBritishEnglishResourceStringRule
	instanceVariableNames:'lastClass resourcesPerLanguage missingLanguages combinations'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Smalltalk/X'
!

!RBBritishEnglishResourceStringRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) 2017 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBBritishEnglishResourceStringRule methodsFor:'accessing'!

group
    ^ 'Miscellaneous - Smalltalk/X'

    "Modified: / 28-11-2017 / 16:06:18 / cg"
!

name
    ^ 'UK-English (instead of US-english) in resource-translated strings'

    "Modified: / 28-11-2017 / 16:03:25 / cg"
!

rationale
    ^ self 
        appendSearchStringsResultAs:'Questionable word(s):' 
        toRationale: 
'Checks for UK-english words in resource-translated strings (for example: "colour" vs. "color").
Each string should be in US-english spelling, to avoid the need for multiple entries in
the resource files and a common spelling inside the application. 
Any UK-english texts should then be generated by translating the US-word in a separate "en_uk.rs" file.
<P>
In addition, some other common spelling errors are checked for (eg. "dont" vs. "don''t" and "referer" vs. "referrer").
<P>
Notice that this does not look for such spelling errors in method selectors,
which are much harder to change without probably breaking existing apps.
Therefore, better leave those unchanged.'

    "Modified: / 29-11-2017 / 11:15:16 / cg"
! !

!RBBritishEnglishResourceStringRule methodsFor:'checking'!

checkMethod: aContext 
    "a quick rejector (if there is any string-literal in the source), to avoid many parses (because parsing is very slow)"

    |source method spec|

    source := aContext sourceCode.
    (source includes: $') ifFalse:[^ self]. 

    method := aContext compiledMethod.
    (method hasAnyResource:#(canvas menu)) ifTrue:[
        ObsoleteMethodCallWarning ignoreIn:[
            spec := method valueWithReceiver:nil arguments:nil.
        ].
        
        spec isCollection ifFalse:[
            Transcript showCR:'ouch: %1 has resource-annotation, but does not return a literal spec array'
                         with:(method whoString).
        ] ifTrue:[
            spec flatDo:[:element |
                element isSymbol ifFalse:[
                    element isString ifTrue:[
                        (self checkString:element) notNil ifTrue:[
                            result addSearchAction:(RBEnvironmentStringSearchAction 
                                                        searchString:element inMethod:method).
                            "/ kept for the rationale
                            result addSearchString:element.
                        ]
                    ]
                ]
            ]
        ]
    ].

    ^ super checkMethod:aContext

    "Modified (format): / 28-11-2017 / 16:05:22 / cg"
    "Modified: / 28-05-2019 / 08:15:33 / Claus Gittinger"
!

checkString:aString
    "check aString for correct us-english words;
     return nil if ok, or the word in question if not.
     here, the emphasis is on words which are different in english as opposed to us-english.
     We want us-english only, to avoid a need for multiple/conflicting translations,
     i.e. to simpify the resource file structure.
    "

    "/ self class classResources string:'cant'.

    #(
        'cancell'       "/ us: canceling, canceled
        'colour'        "/ us: color
        'centre'        "/ us: center
        'favourite'     "/ us: favorite
        'grey'          "/ us: gray
        'advisor'       "/ us: adviser
        'realise'       "/ us: realize
        'realising'
        'realisation'
        'practise'      "/ us: practice

        "/ typos
        'cant'          "/ should in any case be: can't
        'isnt'          "/ should in any case be: isn't   
        'wasnt'         "/ should in any case be: wasn't   
        'wont'          "/ should in any case be: won't  
        'shoulnt'       "/ should in any case be: should't  
        'wouldnt'       "/ should in any case be: would't  

        'referer'       "/ should be referrer
    ) do:[:each |
        (aString includesString:each caseSensitive:false) ifTrue:[^ each].
    ].    
    ^ nil

    "Created: / 28-11-2017 / 16:55:16 / cg"
    "Modified: / 29-11-2017 / 11:36:32 / cg"
!

checkStringNode:aStringNode
    "check aString for correct us-english words;
     return nil if ok, or the word in question if not.
     here, the emphasis is on words which are different in english as opposed to us-english.
     We want us-english only, to avoid a need for multiple/conflicting translations,
     i.e. to simpify the resource file structure.
    "
     
    ^ self checkString:aStringNode value

    "Created: / 28-11-2017 / 16:55:16 / cg"
    "Modified: / 29-11-2017 / 11:36:32 / cg"
! !

!RBBritishEnglishResourceStringRule methodsFor:'initialization'!

initialize
        super initialize.
        self matcher
                matchesAnyOf: #(
                        '(`@rcvr nlsText:`#label)'
                        '(`@rcvr nlsText:`#label with:`@arg1)'
                        '(`@rcvr nlsText:`#label with:`@arg1 with:`@arg2)'
                        '(`@rcvr nlsText:`#label with:`@arg1 with:`@arg2 with:`@arg3)'
                        '(`@rcvr nlsText:`#label withArgs:`@args)'
                        '(`@rcvr nlsText:`#label withArguments:`@args)'
                        '(`@rcvr nlsTranslated:`#label)'
                        '(`@rcvr nlsTitle:`#label)'
                        '(`@rcvr nlsTtitle:`#label with:`@arg1)'

                        '(`@rcvr informationTranslated:`#label)'
                        '(`@rcvr informationTranslated:`#label with:`@arg1)'

                        '(`@resources string:`#label)'
                        '(`@resources string:`#label with:`@arg1)'
                        '(`@resources string:`#label with:`@arg1 with:`@arg2)'
                        '(`@resources string:`#label with:`@arg1 with:`@arg2 with:`@arg3)'
                        '(`@resources string:`#label with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4)'
                        '(`@resources string:`#label with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4 with:`@arg5)'
                        '(`@resources string:`#label with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4 with:`@arg5 with:`@arg6)'
                        '(`@resources string:`#label with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4 with:`@arg5 with:`@arg6 with:`@arg7)'
                        '(`@resources string:`#label withArgs:`@args)'
                        '(`@resources string:`#label withArguments:`@args)'

                        '(`@resources stringWithCRs:`#label)'
                        '(`@resources stringWithCRs:`#label with:`@arg1)'
                        '(`@resources stringWithCRs:`#label with:`@arg1 with:`@arg2)'
                        '(`@resources stringWithCRs:`#label with:`@arg1 with:`@arg2 with:`@arg3)'
                        '(`@resources stringWithCRs:`#label with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4)'
                        '(`@resources stringWithCRs:`#label with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4 with:`@arg5)'
                        '(`@resources stringWithCRs:`#label with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4 with:`@arg5 with:`@arg6)'
                        '(`@resources stringWithCRs:`#label with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4 with:`@arg5 with:`@arg6 with:`@arg7)'
                        '(`@resources stringWithCRs:`#label withArgs:`@args)'
                        '(`@resources stringWithCRs:`#label withArguments:`@args)'

                        '(`@resources string:`#label default:`#def)'
                        '(`@resources string:`#label default:`#def with:`@arg1)'
                        '(`@resources string:`#label default:`#def with:`@arg1 with:`@arg2)'
                        '(`@resources string:`#label default:`#def with:`@arg1 with:`@arg2 with:`@arg3)'
                        '(`@resources string:`#label default:`#def with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4)'
                        '(`@resources string:`#label default:`#def withArgs:`@args)'
                        '(`@resources string:`#label default:`#def withArguments:`@args)'
                )
                do: [:node :answer :owner|
                    |badWordOrNil stringNode|

                    badWordOrNil := nil.
                    stringNode := node arguments first.

                    stringNode isLiteralNode ifTrue:[
                        stringNode value isString ifFalse:[
                            Transcript showCR:'[Lint] invalid (non-string) argument in resource translation'.
                            Transcript showCR:('[Lint] in: ' , (owner currentSearchContext compiledMethod whoString)).
                        ] ifTrue:[    
                            badWordOrNil := (self checkStringNode:stringNode).
                        ].
                    ].
                    badWordOrNil notNil ifTrue:[
                        result addSearchAction:(RBEnvironmentStringSelectAction 
                                                        selectFrom:stringNode start to:stringNode stop 
                                                        inMethod:owner currentSearchContext compiledMethod).
                        "/ kept for the rationale
                        result addSearchString:badWordOrNil.
                        node
                    ] ifFalse:[
                        answer
                    ].
                ]

    "Modified: / 04-04-2017 / 10:12:53 / stefan"
    "Modified (comment): / 29-11-2017 / 11:38:36 / cg"
! !

!RBBritishEnglishResourceStringRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

