"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011-2013 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011-2013 Claus Gittinger (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2022 LabWare

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBBlockLintRule subclass:#RBClassNotReferencedRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Block Rules'
!

!RBClassNotReferencedRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011-2013 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011-2013 Claus Gittinger (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2022 LabWare

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBClassNotReferencedRule methodsFor:'accessing'!

group
        ^ 'Unnecessary Code'
!

name
	^ 'Class not referenced'
!

rationale
        ^ 'Check if a class is referenced either directly or indirectly by a symbol. 
<BR>If a class is not referenced, it can be removed. 
<P>Please, be <B>very very careful</B>: many classes are referenced via their namespace, 
some by enumerating subclasses and a few even by constructing their name dynamically at runtime.
So it is usually better to first put breakpoints on the class''s methods, 
and run the application before, to detect any such references.'

    "Modified: / 30-12-2011 / 12:47:53 / cg"
!

resultClass
    "I will return the set of failing classes"

    ^ ClassEnvironment
!

severity
        ^ SeverityInformation
! !

!RBClassNotReferencedRule methodsFor:'running'!

checkClass: aContext
    | cls environment |
    
    cls := aContext selectedClass.
    cls isMeta ifTrue: [ ^ self ].

    cls subclasses notEmpty ifTrue: [ ^ self ].
    (cls includesBehavior: TestCase) ifTrue: [ ^ self ].
    (cls isProjectDefinition) ifTrue: [ ^ self ].

    "/ environment := aContext environment
    environment := BrowserEnvironment new.
    environment classesDo: [ :eachCls |
        cls isSharedPool ifTrue: [
            (eachCls sharedPoolNames includes: cls name) ifTrue: [ 
                ^ self "/ Class is referenced
            ].
        ].

        environment selectorsForClass: eachCls do: [ :eachSelector |
            | eachMethod |

            eachMethod := eachCls compiledMethodAt: eachSelector.
            (eachMethod referencesGlobal: cls) ifTrue: [ 
                ^ self  "/ Class is referenced    
            ].
        ].        

        environment selectorsForClass: eachCls theMetaclass do: [ :eachSelector |
            | eachMethod |

            eachMethod := eachCls theMetaclass compiledMethodAt: eachSelector.
            (eachMethod referencesGlobal: cls) ifTrue: [ 
                ^ self  "/ Class is referenced    
            ].
        ].        
    ].

    result
        addClass: cls;
        addClass: cls class

    "Modified: / 18-08-2022 / 12:47:51 / Jan Vrany <jan.vrany@labware.com>"
! !

!RBClassNotReferencedRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

