"
 Copyright (C) 2017 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBCodeDuplicationRule subclass:#RBCodeDuplicationRule2
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Block Rules'
!

!RBCodeDuplicationRule2 class methodsFor:'documentation'!

copyright
"
 Copyright (C) 2017 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBCodeDuplicationRule2 methodsFor:'accessing'!

group
        ^ 'Unnecessary Code'
!

name
    ^ 'Method''s functionality is already provided by another method'

    "Modified: / 26-02-2017 / 15:01:12 / cg"
!

rationale
        ^ 'Check for methods which duplicate an already existing method''s functionality.
<br>This can be any method here or inherited which performs the same function,
either with the same message selector, or a different one.
<p>If an implementation for the same-selector is inherited, the method can be removed.
<br>For a different-selector method, the code here may either be changed to just send
the other message, or the senders of it can be changed to send the other message (and the method here removed).
<p>Be careful:<br>often methods have to be implemented this way, to implement the required protocol,
and you cannot simply change the senders.
<br>Also, duplicates might be present for backward or Smalltalk dialect compatibility.'

    "Modified: / 07-03-2017 / 17:59:35 / cg"
    "Modified: / 02-11-2018 / 10:20:45 / Claus Gittinger"
!

severity
        ^ SeverityInformation
! !

!RBCodeDuplicationRule2 methodsFor:'running'!

checkMethod: aContext
    |cls selector methodHere firstFound literalsHere parseTreeHere isJustReturningSomething|

    cls := aContext selectedClass.
    cls := cls realClass.
    cls isNil ifTrue:[^ self].
    
    selector := aContext selector.

    cls isMeta ifTrue:[
        "/ these are meant to be empty and only contain different comments...
        ( #(
            documentation version examples
            copyright history
            initialize "/ because that is not invoked if only inherited
           ) includes:selector) 
        ifTrue:[
            ^ self
        ].
        "/ these implement subPackages etc., which I want to have as documentation
        cls isProjectDefinition ifTrue:[^ self].
    ].

    methodHere := cls compiledMethodAt:selector.
    "/ ignore subclassResponsibility and other error reporting methods
    (methodHere sendsAnySelector:#( 
                    #subclassResponsibility #subclassResponsibility: 
                    #implementedBySubclass  "/ ST/V code uses this
                    #shouldNotImplement #shouldNeverBeSent #shouldImplement #shouldImplement:
                    #notYetImplemented) 
    ) ifTrue:[^ self].

    literalsHere := methodHere literals.
    parseTreeHere := RBParser
                        parseMethod:methodHere source ? ''
                        onError: [:aString :position | nil "ignore any error"].
    parseTreeHere isNil ifTrue:[^ self].

    "/ ignore simple single send methods, which forward
    parseTreeHere allMessageNodes size < 2 ifTrue:[^ self].

    "/ ignore simple value returns
    isJustReturningSomething := (ParseTreeSearcher isJustReturningSomething)
                                    executeTree: parseTreeHere initialAnswer: false.
    "/ isJustReturningSomething ifTrue: [^self].
    
    "/ ignore subclassResponsibility methods
    "/ ((ParseTreeSearcher isSubclassResponsibility)
    "/    executeTree: parseTreeHere initialAnswer: false) ifTrue: [^self].
        
    firstFound := true.
    cls withAllSuperclasses do:[:candidateClass |
        candidateClass methodDictionary keysAndValuesDo:[:selThere :methodThere |
            |parseTreeThere|
            
            (methodThere ~~ methodHere) ifTrue:[
                (isJustReturningSomething not or:[ selThere == selector ]) ifTrue:[
                    "/ for just-returning-something methods, only check inherited ones
                    (literalsHere sameContentsAs: methodThere literals) ifTrue:[
                        parseTreeThere := RBParser
                                            parseMethod:methodThere source ? ''
                                            onError: [:aString :position | nil "ignore any error"].
                        parseTreeThere notNil ifTrue:[
                            (self class
                                    hasParseTree:parseTreeHere 
                                    sameSemanticAs:parseTreeThere ignoreAnnotations:false
                            ) ifTrue:[  
                                "/ these are allowed to be empty and only contain different comments...
                                (cls isMeta 
                                    and:[methodHere category = 'documentation'
                                    and:[methodThere category = 'documentation'
                                    and:[isJustReturningSomething
                                ]]]) ifFalse:[    

                                    otherImplementationsPerMethod isNil ifTrue:[
                                        otherImplementationsPerMethod := Dictionary new
                                    ].
                                    (otherImplementationsPerMethod at:methodHere ifAbsentPut:[Set new]) add:methodThere.
                                    firstFound ifTrue:[
                                        result addClass:cls selector:selector. 
                                    ]. 
                                    firstFound := false
                                ].    
                            ].    
                        ].    
                    ].    
                ].    
            ].    
        ].
    ].

    "Modified: / 13-07-2017 / 15:55:04 / cg"
! !

!RBCodeDuplicationRule2 class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

