"
 Copyright (C) 2013 Claus Gittinger
 Copyright (C) 2022 LabWare

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBBlockLintRule subclass:#RBContainsSmalltalkXEOLCommentRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Smalltalk/X'
!

!RBContainsSmalltalkXEOLCommentRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) 2013 Claus Gittinger
 Copyright (C) 2022 LabWare

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBContainsSmalltalkXEOLCommentRule class methodsFor:'accessing'!

ignoredPackages
    "don't complain about those packages"

    ^ #(
        'stx:*'
        'exept:*'
    )

    "Created: / 26-02-2017 / 16:20:40 / cg"
!

tagsBasic
    "Return an array of tags (symbols) of given rule. Any rule may
     arbitrary number of tags. Tag values are not predefined/fixed.
     See class documentation for list of common tags"

    ^ #( 
        portability "/ code is not portable
        stx         "/ code is Smalltalk/X specific
        vse         "/ code does not work in vse
        vw          "/ code does not work in vw
    )

    "Created: / 10-01-2013 / 11:13:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (comment): / 28-10-2014 / 12:02:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 07-05-2020 / 12:40:55 / cg"
! !

!RBContainsSmalltalkXEOLCommentRule class methodsFor:'private'!

eolAndTokenCommentsIn: source
    "cg: moved to class side as utility;
     it is also used by the VW portability rules"

    ^ OrderedCollection streamContents: [ :s | 
        self eolAndTokenCommentsIn: source do: [ :comment | s nextPut: comment ]  
    ]

    "Created: / 17-10-2014 / 00:27:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (comment): / 15-05-2020 / 14:10:44 / cg"
!

eolAndTokenCommentsIn: source do: block 
    "cg: moved to class side as utility;
     it is also used by the VW portability rules"

    | scanner |

    scanner := RBScanner on: source readStream.
    [ 
        | comments |

        comments := scanner getComments.
        comments notEmptyOrNil ifTrue:[ 
            comments do:[:each | 
                (each isEOLComment or:[ each isTokenComment ]) ifTrue:[ 
                    block value: each.
                ].
            ].
        ].
        scanner next isEOF
    ] whileFalse.

    "Created: / 16-10-2014 / 14:33:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (comment): / 15-05-2020 / 14:10:40 / cg"
! !

!RBContainsSmalltalkXEOLCommentRule methodsFor:'accessing'!

changes
    | builder |

    builder := CompositeRefactoryChange named: self name.    
    result classesDo:[ :cls |
        result selectorsForClass: cls do: [ :selector |
            | method source comments |

            source := (method := cls compiledMethodAt: selector) source.
            comments := self class eolAndTokenCommentsIn: source.
            comments notEmptyOrNil ifTrue:[ 
                comments reverseDo:[:comment |
                    comment isEOLComment ifTrue:[ 
                        source := 
                            (source copyTo: comment start - 1),
                            '"',
                            (comment characters copyFrom: 3),
                            '"',
                            (source copyFrom: comment last + 1)
                    ].
                ].
                builder compile: source in: cls classified: method category
            ].
        ].
    ].
    ^ builder changes

    "Created: / 17-10-2014 / 00:19:51 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 15-05-2020 / 14:09:00 / cg"
    "Modified: / 22-09-2022 / 17:26:12 / Jan Vrany <jan.vrany@labware.com>"
!

group
        ^ 'Portability Issues'
!

name
        ^ 'Contains ST/X EOL- or token-comments'
!

nameOfFix
    ^ 'Change EOL comments to standard Smalltalk comments'
!

rationale
        ^ 'Checks for unportable dialect specific End-of-Line or Token delimited comments. 
These are only supported by ST/X. 
<P>You will have to change them into regular comments,
when porting this code to other Smalltalk dialects.'
!

severity
        ^ SeverityInformation
! !

!RBContainsSmalltalkXEOLCommentRule methodsFor:'redefinable utilities'!

resultSelectionIntervalsForSource:aString tree:aTree in: aClass do:aBlock
    "the lint highlighter wants me to provide highlighting intervals."

    self class eolAndTokenCommentsIn: aString do:[:comment | 
        aBlock value: comment sourceInterval
    ]

    "Created: / 02-12-2014 / 11:14:46 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 15-05-2020 / 14:09:29 / cg"
! !

!RBContainsSmalltalkXEOLCommentRule methodsFor:'running'!

checkMethod: aContext
    | cls source |

    aContext isNil ifTrue:[
        "/ My guess: happens when two rules are executed in parallel (seems to no longer happen)
        self breakPoint:#cg.      
        ^ self.
    ].   

    cls := aContext selectedClass.

    "/ ignore packages stx and exept.
    (self class ignoredPackages contains:[:pattern | pattern match:cls package]) ifTrue:[^ self].
    
    source := aContext sourceCode.
    "/ a quick rejector

    ((source includesString:'"/') or:[ (source includesString:'"<<') ]) ifFalse:[^ self].
    self class eolAndTokenCommentsIn: source do:[:each | 
        self 
            addClass: cls 
            selector: aContext selector 
            to: result.
        ^ self.
    ].

    "Created: / 16-10-2014 / 14:29:13 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 15-05-2020 / 14:08:39 / cg"
! !

!RBContainsSmalltalkXEOLCommentRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

