"{ Encoding: utf8 }"

"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011-2013 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011-2013 Claus Gittinger (porting, Smalltalk/X rules & tweaks)

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBBlockLintRule subclass:#RBJustSendsSuperRule
	instanceVariableNames:'matcher'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Block Rules'
!

!RBJustSendsSuperRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011-2013 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011-2013 Claus Gittinger (porting, Smalltalk/X rules & tweaks)

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBJustSendsSuperRule methodsFor:'accessing'!

group
        ^ 'Unnecessary Code'
!

name
	^ 'Method just sends super message'
!

rationale
        ^ 'Check for methods that just forward the message to its superclass.<br>
These methods can be removed (except to put a breakpoint for instances of a specific class).
<P>
<B>Warning:</B>
<br>in ST/X, there are a few exceptions to this:
<BR>the initialize-message on the class side (which is sent by the runtime system at startup time)
will only be sent if implemented in the class (not if inherited).
This is done to avoid multiple calls to a superclasses initialization code.
<P>
Thus be careful, if you see "super initialize" on the class side.
Do not remove them without checking for code in a superclass which initializes class-instance variables.
If you remove that method, those variables might no longer be initialized in the subclass.
<P>
Class-initialize methods are skipped by this rule.'

    "Modified: / 14-02-2017 / 09:13:26 / cg"
!

severity
        ^ SeverityInformation
! !

!RBJustSendsSuperRule methodsFor:'formatting'!

resultSelectionIntervalsForSource:string tree:tree in:class do:aBlock
    "evaluate aBlock for all text-intervals 
     (used by the highlighter to underwave affected code fragments).
     here we highlight the selector (any other suggestion?)"

    self resultSelectionIntervalsForSelectorInSource:string tree:tree in:class do:aBlock

    "Created: / 26-02-2017 / 23:15:37 / cg"
! !

!RBJustSendsSuperRule methodsFor:'initialization'!

initialize
    super initialize.
    matcher := ParseTreeSearcher justSendsSuper
! !

!RBJustSendsSuperRule methodsFor:'running'!

checkMethod: aContext
    |cls selector parseTree source|

    cls := aContext selectedClass.
    selector := aContext selector.

    "/ for the class>>initialize method, this makes a difference
    ((selector == #initialize )
    or:[selector == #updateStyleCache])ifTrue:[
        cls isMeta ifTrue:[^ self].
    ].
    
    "a quick rejector, to avoid many parses (because parsing is very slow)"
    source := aContext sourceCode.
    (source includesString: 'super') ifFalse:[^ self]. 
    
    parseTree := aContext parseTree.
    (parseTree isPrimitive not 
    and: [ matcher executeMethod: parseTree initialAnswer: false ]) ifTrue: [ 
        result addClass: cls selector: selector 
    ]

    "Modified: / 14-02-2017 / 09:20:23 / cg"
    "Modified (format): / 26-02-2017 / 14:22:10 / cg"
    "Modified: / 28-05-2019 / 08:16:09 / Claus Gittinger"
! !

!RBJustSendsSuperRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

