"{ Encoding: utf8 }"

"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBBlockLintRule subclass:#RBMethodModifierSuperRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Block Rules'
!


!RBMethodModifierSuperRule methodsFor:'accessing'!

group
	^ 'Bugs'
!

name
	^ 'Super call required'
!

rationale
    ^ 'Checks that methods marked with &lt;modifier: #super&gt; are always called when overridden.
In other words: a "super"-message is missing in the code.'

    "Modified: / 08-02-2017 / 00:11:14 / cg"
!

severity
	^ #error
! !

!RBMethodModifierSuperRule methodsFor:'formating'!

resultSelectionIntervalsForSource:string tree:tree in:class do:aBlock
    "evaluate aBlock for all text-intervals 
     (used by the highlighter to underwave affected code fragments).
     here we highlight the selector (any other suggestion?)"

    self resultSelectionIntervalsForSelectorInSource:string tree:tree in:class do:aBlock

    "Created: / 08-02-2017 / 00:05:02 / cg"
    "Modified: / 26-02-2017 / 23:14:29 / cg"
! !

!RBMethodModifierSuperRule methodsFor:'running'!

checkMethod: aContext
    |selectedClass selector implementors topAnnotatedMethod|

    "/ this is expensive - only do it, if a super-method has this annotation
    "/ (aContext superMessages includes: aContext selector) ifTrue: [ ^ self ].

    selectedClass := aContext selectedClass.
    selectedClass superclass isNil ifTrue:[^ self].

    "/ selectedClass == BookmarkBar ifTrue:[self halt].
    
    selector := aContext selector.
    
    implementors := OrderedCollection with:(selectedClass compiledMethodAt:selector).
    topAnnotatedMethod := nil.
    
    selectedClass superclass withAllSuperclassesDo:[:superClass |
        |implMethod|
        
        (implMethod := superClass compiledMethodAt:selector) notNil ifTrue:[
            implementors add:implMethod.
            
            implMethod 
                annotationsAt:#modifier:
                do:[:each | 
                    (each arguments first = #super) ifTrue:[
                        "/ yes - has the annotation
                        topAnnotatedMethod := implMethod.
                    ].    
                ].    
        ]
    ].
    topAnnotatedMethod notNil ifTrue:[
        "/ all methods along the superclass chain MUST send super,
        "/ up to the topMost annotated method.
        "/ however, this will be irritating in the browser (showing a method which is not guilty)
        "/ therefore, only check the currentMethod
        {selectedClass compiledMethodAt:selector}
        "/ implementors 
        do:[:each |
            each == topAnnotatedMethod ifTrue:[^ self]. "/ done
            
            (each superMessages includes:selector) ifFalse:[
                "/ although the error may be in an inherited method, we report it here.
                "/ user has to check this
                result addClass:selectedClass selector:selector.
                ^ self
            ].    
        ].    
    ].

    "Modified: / 04-08-2011 / 21:28:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 08-02-2017 / 00:42:53 / cg"
! !

!RBMethodModifierSuperRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

