"
 Copyright (C) 2017 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBParseTreeLintRule subclass:#RBMisplacedCriticalReturningVariableOrConstant
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Smalltalk/X'
!

!RBMisplacedCriticalReturningVariableOrConstant class methodsFor:'documentation'!

copyright
"
 Copyright (C) 2017 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBMisplacedCriticalReturningVariableOrConstant methodsFor:'accessing'!

group
    ^ 'Performance - Smalltalk/X'

    "Created: / 07-02-2017 / 11:27:13 / cg"
!

name
    ^ 'Possibly unneeded critical block'
!

rationale
        ^ 'Checks for unneeded critical blocks, which simply return a constant or variable.
<BR>Code like
<PRE><CODE>    sema critical:[ ^ variable ]</CODE></PRE>
(i.e. returning the value of a variable from inside a critical block) do not need the critical section.
<P>
With the critical section, you will either get the value before someone else has modified it,
or the value after the modification. Which is exactly what you get without the critical section.
<P>
<b>Attention:</b>
There are situations in which such code is required to protect bad code
from gettinging invalid values or
disturbing another critical sections computation:
<ol>
 <li>if someone else leaves an invalid value temporarily inside the variable during a longer
 computation: 
 <UL>
  <LI>fetches the value,
  <LI>changes it temporarily to a value which should not be seen elsewhere,
  <LI>performs more computations,
  <LI>writes the final (valid) value into the variable
 </UL>
 <LI>if someone reads the value multiple times:
 <UL>
  <LI>fetches the value (typically: a collection)
  <LI>operates on it (typically: adds or removes elements)
  <LI>fetches the value again
  <LI>operates on it (typically: adds or removes more elements)
 </UL>
</ol>

However, both cases are very bad code, which should be fixed there, 
not by trying to workaround that other code''s mistake here.
For situation 1) in the above, change the other critical section
to not write the invalid intermediate value into the variable,
but keeps it inside a private temporary, and only write the final value into the destination variable.
<BR>
For situation 2), change the other critical section to fetch the variable once into a temporary
and operate on that temporary instead of the shared variable.
<P>
The same argument goes for stores into the protected variable: the only reason for a critical region
would be to protect the other critical region''s (the bad code''s) temporary value.
<P>
Of course, constant value returns never need a critical region around.'

    "Created: / 07-02-2017 / 11:27:13 / cg"
    "Modified: / 08-02-2017 / 12:44:03 / stefan"
! !

!RBMisplacedCriticalReturningVariableOrConstant methodsFor:'initialization'!

initialize
        super initialize.
        self matcher 
                matchesAnyOf: #(
                    '`@e critical:[ `v ]'
                    '`@e critical:[ `#n ]'
                    '`@e critical:[ ^ `v ]'
                    '`@e critical:[ ^ `#n ]'
                    '`@e critical:[ `v1 := `v2 ]'
                    '`@e critical:[ `v1 := `#n ]'
                    '`@e critical:[ `v isNil ]'
                    '`@e critical:[ `v notNil ]'
                    '`@e critical:[ `v == `#n ]'
                    '`@e critical:[ `v = `#n ]'
                    '`@e critical:[ `v ~~ `#n ]'
                    '`@e critical:[ `v ~= `#n ]'
                    '`@e critical:[ `v > `#n ]'
                    '`@e critical:[ `v >= `#n ]'
                    '`@e critical:[ `v < `#n ]'
                    '`@e critical:[ `v <= `#n ]'
                   )

                do: [ :node :answer | node ]

    "Created: / 07-02-2017 / 11:27:13 / cg"
! !

!RBMisplacedCriticalReturningVariableOrConstant class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

