"
 Copyright (C) 2017 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBBlockLintRule subclass:#RBNonAbstractClassWithSubclassResponsibilityMethodsRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Block Rules'
!

!RBNonAbstractClassWithSubclassResponsibilityMethodsRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) 2017 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
!

documentation
"
    This rules identifies when several methods in a set of subclasses 
    are defined but not in the superclass. 
    It identifies that an abstract method is missing in the superclass.

"
! !

!RBNonAbstractClassWithSubclassResponsibilityMethodsRule methodsFor:'accessing'!

group
	^ 'Bugs'
!

name
    ^ 'Non-abstract classes should not contain methods marked as subclassResponsibility'

    "Modified: / 10-02-2017 / 10:26:09 / cg"
!

rationale
    ^ 'Checks that a non-abstract class does not contain a method which are marked as subclassResponsibility.
There should usually be either an implementation here, or the class marked as being abstract'

    "Modified: / 10-02-2017 / 10:27:08 / cg"
!

severity
        ^ SeverityWarning

    "Modified: / 10-02-2017 / 10:27:14 / cg"
! !

!RBNonAbstractClassWithSubclassResponsibilityMethodsRule methodsFor:'running'!

checkClass:aContext 
    |theClass|

    theClass := aContext selectedClass.
    theClass theNonMetaclass isAbstract ifTrue:[^ self].

    theClass selectorsAndMethodsDo:[:selector :method | 
        (method isSubclassResponsibility) ifTrue:[
            self 
                addClass:theClass
                selector:selector                        
                to:result
        ].
    ].

    "Modified (format): / 22-01-2013 / 18:42:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 16-07-2017 / 11:31:23 / cg"
! !

!RBNonAbstractClassWithSubclassResponsibilityMethodsRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

