"{ Encoding: utf8 }"

"
 Copyright (C) 2018 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBParseTreeLintRule subclass:#RBObiousSpellingErrorsInResourceStringRule
	instanceVariableNames:'lastClass resourcesPerLanguage missingLanguages combinations'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Smalltalk/X'
!

!RBObiousSpellingErrorsInResourceStringRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) 2018 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
!

documentation
"
    looks for common and obvious spelling errors in labels, resource string, menu items etc.
    By no means, this is complete - it is used as a quick helper for
    UI programmers in our company (exept) and contains checks for typical and already
    occurred spelling errors.

    [author:]
        cg

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!RBObiousSpellingErrorsInResourceStringRule methodsFor:'accessing'!

group
    ^ 'Miscellaneous - Smalltalk/X'

    "Modified: / 28-11-2017 / 16:06:18 / cg"
!

name
    ^ 'Obvious spelling errors in resource-translated strings'

    "Modified: / 28-11-2017 / 16:03:25 / cg"
!

rationale
    ^ self 
        appendSearchStringsResultAs:'Possibly spelling error(s):' 
        toRationale: 
'Checks for some common and obvious spelling errors in resource-translated strings.
Notice, that the set of checks done here is in no way complete.
However, it will cover common and already encountered errors as they appear
to help UI programmers to not make the same mistake again.
Please add more to the check list as errors are detected.
'
    "Modified: / 29-11-2017 / 11:15:16 / cg"
! !

!RBObiousSpellingErrorsInResourceStringRule methodsFor:'checking'!

checkMethod: aContext 
    |source spec method ast stats firstStat lit anyFound|

    "a quick rejector (if there is any array+string-literal in the source), 
     to avoid many parses (because parsing is relatively slow)"
    source := aContext sourceCode.
    (source includes: $') ifFalse:[^ self].
    (source includes: $#) ifFalse:[^ self].

    method := aContext compiledMethod.
    (method hasAnyResource:#(canvas menu)) ifTrue:[
        anyFound := false.

        ast := method parseTree.
        ast notNil ifTrue:[
            (stats := ast body statements) size == 1 ifTrue:[
                (firstStat := stats first) isReturn ifTrue:[
                    (lit := firstStat value) isLiteralArray ifTrue:[
                        spec := lit value.
                        spec flatDoWithParent:[:element :parent |
                            |idx|

                            (element isString and:[element isSymbol not]) ifTrue:[
                                "/ we are in a spec: only care for strings after a label: elements
                                (((idx := parent identityIndexOf:element) > 1) 
                                and:[ (parent at:(idx-1)) = #'label:' ]) ifTrue:[
                                    (self checkString:element) notNil ifTrue:[
                                        anyFound := true.
                                        result addSearchAction:(RBEnvironmentStringSearchAction 
                                                                    searchString:element inMethod:method).
                                        "/ kept for the rationale
                                        result addSearchString:element.
                                    ]
                                ]
                            ]
                        ]
                    ]
                ].
            ].
        ].

        anyFound ifTrue:[
            result addClass:(aContext selectedClass) selector:(aContext selector) 
        ].
    ].

    ^ super checkMethod:aContext

    "Modified (format): / 28-11-2017 / 16:05:22 / cg"
!

checkString:aString
    "check aString for correct us-english words;
     return nil if ok, or the word in question if not.
     here, the emphasis is on words which are different in english as opposed to us-english.
     We want us-english only, to avoid a need for multiple/conflicting translations,
     i.e. to simpify the resource file structure.
    "
     
    "/ self class classResources string:'cant'.

    #(
        "/ typos
        'cant'          "/ should in any case be: can't
        'isnt'          "/ should in any case be: isn't   
        'wasnt'         "/ should in any case be: wasn't   
        'wont'          "/ should in any case be: won't  
        'shoulnt'       "/ should in any case be: should't  
        'wouldnt'       "/ should in any case be: would't  

        'referer'           "/ should be referrer
        'does exists'       "/ should be exist
        'does not exists'   "/ should be exist
    ) do:[:each |
        (aString includesString:each caseSensitive:false) ifTrue:[^ each].
    ].    
    ^ nil

    "Created: / 28-11-2017 / 16:55:16 / cg"
    "Modified: / 29-11-2017 / 11:36:32 / cg"
    "Modified: / 28-05-2019 / 08:12:12 / Claus Gittinger"
!

checkStringNode:aStringNode
    "check aStringNode for correct us-english words;
     return nil if ok, or the word in question if not.
     here, the emphasis is on words which are different in english as opposed to us-english.
     We want us-english only, to avoid a need for multiple/conflicting translations,
     i.e. to simpify the resource file structure.
    "
     
    ^ self checkString:(aStringNode value)

    "Created: / 28-11-2017 / 16:55:16 / cg"
    "Modified: / 29-11-2017 / 11:36:32 / cg"
! !

!RBObiousSpellingErrorsInResourceStringRule methodsFor:'initialization'!

initialize
        super initialize.
        self matcher
                matchesAnyOf: #(
                        '(`@rcvr nlsText:`#label)'
                        '(`@rcvr nlsText:`#label with:`@arg1)'
                        '(`@rcvr nlsText:`#label with:`@arg1 with:`@arg2)'
                        '(`@rcvr nlsText:`#label with:`@arg1 with:`@arg2 with:`@arg3)'
                        '(`@rcvr nlsText:`#label withArgs:`@args)'
                        '(`@rcvr nlsTranslated:`#label)'
                        '(`@rcvr nlsTitle:`#label)'
                        '(`@rcvr nlsTtitle:`#label with:`@arg1)'

                        '(`@rcvr informationTranslated:`#label)'
                        '(`@rcvr informationTranslated:`#label with:`@arg1)'
                        '(`@rcvr warning:`#label)'
                        '(`@rcvr warning:`#label)'
                        '(`@rcvr warning:`#label with:`@arg)'
                        '(`@rcvr warning:`#label with:`@arg1 with:`@arg2)'

                        '(`@resources string:`#label)'
                        '(`@resources string:`#label with:`@arg1)'
                        '(`@resources string:`#label with:`@arg1 with:`@arg2)'
                        '(`@resources string:`#label with:`@arg1 with:`@arg2 with:`@arg3)'
                        '(`@resources string:`#label with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4)'
                        '(`@resources string:`#label with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4 with:`@arg5)'
                        '(`@resources string:`#label with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4 with:`@arg5 with:`@arg6)'
                        '(`@resources string:`#label with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4 with:`@arg5 with:`@arg6 with:`@arg7)'
                        '(`@resources string:`#label withArgs:`@args)'

                        '(`@resources stringWithCRs:`#label)'
                        '(`@resources stringWithCRs:`#label with:`@arg1)'
                        '(`@resources stringWithCRs:`#label with:`@arg1 with:`@arg2)'
                        '(`@resources stringWithCRs:`#label with:`@arg1 with:`@arg2 with:`@arg3)'
                        '(`@resources stringWithCRs:`#label with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4)'
                        '(`@resources stringWithCRs:`#label with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4 with:`@arg5)'
                        '(`@resources stringWithCRs:`#label with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4 with:`@arg5 with:`@arg6)'
                        '(`@resources stringWithCRs:`#label with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4 with:`@arg5 with:`@arg6 with:`@arg7)'
                        '(`@resources stringWithCRs:`#label withArgs:`@args)'

                        '(`@resources string:`#label default:`#def)'
                        '(`@resources string:`#label default:`#def with:`@arg1)'
                        '(`@resources string:`#label default:`#def with:`@arg1 with:`@arg2)'
                        '(`@resources string:`#label default:`#def with:`@arg1 with:`@arg2 with:`@arg3)'
                        '(`@resources string:`#label default:`#def with:`@arg1 with:`@arg2 with:`@arg3 with:`@arg4)'
                        '(`@resources string:`#label default:`#def withArgs:`@args)'
                        '(`@resources warn:`#label)'
                )
                do: [:node :answer :owner|
                    |badWordOrNil stringNode|

                    badWordOrNil := nil.
                    stringNode := node arguments first.

                    stringNode isLiteralNode ifTrue:[
                        stringNode value isString ifFalse:[
                            Transcript showCR:'[Lint] invalid (non-string) argument in resource translation'.
                            Transcript showCR:('[Lint] in: ' , (owner currentSearchContext compiledMethod whoString)).
                        ] ifTrue:[    
                            badWordOrNil := (self checkStringNode:stringNode).
                        ].
                    ].
                    badWordOrNil notNil ifTrue:[
                        result addSearchAction:(RBEnvironmentStringSelectAction 
                                                        selectFrom:stringNode start to:stringNode stop 
                                                        inMethod:owner currentSearchContext compiledMethod).
                        "/ kept for the rationale
                        result addSearchString:badWordOrNil.
                        node
                    ] ifFalse:[
                        answer
                    ].
                ]

    "Modified: / 04-04-2017 / 10:12:53 / stefan"
    "Modified (comment): / 29-11-2017 / 11:38:36 / cg"
    "Modified: / 28-05-2019 / 08:11:42 / Claus Gittinger"
! !

!RBObiousSpellingErrorsInResourceStringRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

