"{ Encoding: utf8 }"

"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011-2013 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011-2013 Claus Gittinger (porting, Smalltalk/X rules & tweaks)

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBParseTreeLintRule subclass:#RBPrecedenceRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Parse-tree Rules'
!

!RBPrecedenceRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011-2013 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011-2013 Claus Gittinger (porting, Smalltalk/X rules & tweaks)

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBPrecedenceRule methodsFor:'accessing'!

group
	^ 'Miscellaneous'
!

name
        ^ 'Verify if "A + B * C" was meant to be "A + (B * C)" or "A raisedTo:B+C" to be "A raisedTo:(B+C)"'

    "Modified: / 20-06-2017 / 10:30:46 / cg"
!

rationale
        ^ 'Checks for expressions that might be evaluated different than the developer thinks (e.g. from left-to-right).
<P>
For example, "A + B * C" is evaluated as "(A + B) * C" by Smalltalk precedence rules,
which may or may not what the programmer intended. 
<BR>
Also, "A raisedTo:B + C" is evaluated as "A raisedTo:(B + C)" because the "raisedTo:" keyword message has a lower
precedence than the binary "+" message. This is in contrast to the mathematical power operator, and may be overseen, when
formulas are translated into code.
<P>
Please add parentheses to make the intension explicit and avoid misreading by others.'

    "Modified: / 20-06-2017 / 10:37:10 / cg"
! !

!RBPrecedenceRule methodsFor:'initialization'!

initialize
        super initialize.
        self matcher 
                matchesAnyOf: #(
                            '`{:node | node isMessage 
                                and: [node hasParentheses not and: [#(#+ #-) includes: node selectorString]]} * `@C' 
                                
                            '`{:node | node isMessage 
                                and: [node hasParentheses not and: [#(#+ #-) includes: node selectorString]]} / `@C' 
                                
                            '`{:node | node isMessage 
                                and: [node hasParentheses not and: [#(#+ #- #* #/) includes: node selectorString]]} raisedTo: `@C' 
                                
                            '`{:node | node isMessage 
                                and: [node hasParentheses not and: [#(#+ #- #* #/) includes: node selectorString]]} min: `@C' 
                                
                            '`{:node | node isMessage 
                                and: [node hasParentheses not and: [#(#+ #- #* #/) includes: node selectorString]]} max: `@C' 

                            '`@C * `{:node | node isMessage 
                                and: [node hasParentheses not and: [#(#+ #-) includes: node selectorString]]}' 
                                
                            '`@C / `{:node | node isMessage 
                                and: [node hasParentheses not and: [#(#+ #-) includes: node selectorString]]}' 

                            '`@C raisedTo: `{:node | node isMessage 
                                and: [node hasParentheses not and: [#(#+ #- #* #/) includes: node selectorString]]} ' 
                                
                            '`@C min: `{:node | node isMessage 
                                and: [node hasParentheses not and: [#(#+ #- #* #/) includes: node selectorString]]} ' 

                            '`@C max: `{:node | node isMessage 
                                and: [node hasParentheses not and: [#(#+ #- #* #/) includes: node selectorString]]} ' 

                            '`@C @ `{:node | node isMessage 
                                and: [node hasParentheses not and: [#(#+ #- #* #/) includes: node selectorString]]} ' 
                )
                do: [ :node :answer | node ]

    "Modified: / 20-06-2017 / 10:28:27 / cg"
    "Modified: / 13-03-2019 / 19:39:09 / Claus Gittinger"
! !

!RBPrecedenceRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

