"
 Copyright (C) 2019 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBParseTreeLintRule subclass:#RBQuestionableAssignmentRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Parse-tree Rules'
!

!RBQuestionableAssignmentRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) 2019 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBQuestionableAssignmentRule methodsFor:'accessing'!

group
        ^ 'Unnecessary Code'
!

name
        ^ 'Unnecessary double assignment to a variable'

    "Modified: / 13-03-2019 / 17:04:06 / Claus Gittinger"
!

rationale
    |t|

    ^ (t := 'Check for unnecessary assignment to a variable of the form:
<code><pre>    var := expr ifTrue:[ var := ...] ifFalse:[ var := ...]</pre></code>
Remove either the outer assignment or the two inner assignments
to make the code easier to understand and less error prone.
<br>It is recommended to remove the inner assignments.
')

    "Created: / 01-03-2012 / 11:37:16 / cg"
    "Modified: / 13-03-2019 / 17:06:08 / Claus Gittinger"
!

severity
        ^ SeverityInformation
! !

!RBQuestionableAssignmentRule methodsFor:'initialization'!

initialize
        super initialize.
        self matcher 
                matchesAnyOf:#(
                     '`v := `@e ifTrue:[ `v := `@e2] ifFalse:[ `v := `@e3]'
                     '`v := `@e ifFalse:[ `v := `@e2] ifTrue:[ `v := `@e3]'
                     '`v := `@e ifTrue:[ `v := `@e2]'  "/ should add ifFalse:nil, to make this clear
                     '`v := `@e ifFalse:[ `v := `@e2]' "/ should add ifTrue:nil, to make this clear
                     
                     '`v := `@e ifTrue:[ ^ `@e2]'      "/ will not assign if true; will assign nil if false
                     '`v := `@e ifFalse:[ ^ `@e2]'     "/ will not assign if false; will assign nil if true

                     '`@e ifTrue:[ ^ v := `@e2] ifFalse:[ ^ `v := `@e3]'     "/ will not assign
                     '`@e ifFalse:[ ^ v := `@e2] ifTrue:[ ^ `v := `@e3]'     "/ will not assign
                     '`v := `@e ifTrue:[ ^ v := `@e2] ifFalse:[ `v := `@e3]' "/ will not assign if true
                     '`v := `@e ifTrue:[ `v := `@e2] ifFalse:[ ^ v := `@e3]' "/ will not assign if false

                     '`v := `@e ifFalse:[ ^ `@e2] ifTrue:[ `v := `@e3]'      "/ will not assign if false
                     '`v := `@e ifTrue:[ ^ `@e2] ifFalse:[ `v := `@e3]'      "/ will not assign if talse

                     '`v := `@e ifFalse:[ `v := `@e2] ifTrue:[ ^ `@e3]'
                     '`v := `@e ifTrue:[ `v := `@e2] ifFalse:[ ^ `@e3]'
                     
                     '`v := `@e ifFalse:[ ^ `@e2] ifTrue:[ `v := `@e3]'
                     '`v := `@e ifFalse:[ `v := `@e2] ifTrue:[ ^ `@e3]'
                )
                do: [ :node :answer | node ]

    "Modified: / 13-03-2019 / 17:23:59 / Claus Gittinger"
! !

!RBQuestionableAssignmentRule class methodsFor:'documentation'!

version_CVS
    ^ '$Header$'
! !

