"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBParseTreeLintRule subclass:#RBQuestionableIdentifierRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Parse-tree Rules'
!


!RBQuestionableIdentifierRule methodsFor:'accessing'!

group
    ^ 'Intention Revealing'

    "Created: / 03-08-2017 / 14:26:56 / cg"
!

name
    ^ 'Checks for questionable variable names.'

    "Created: / 03-08-2017 / 14:27:21 / cg"
!

rationale
        ^ 'Checks for questionable variable names.
<P>
For example, a name like "l" (lower case "L") is easily read as "1" (the number "one"),
as they look almost the same in most fonts.
Please rename those to something like "len" or "lenOfX", or whatever the variable represents.
<P>
Also, names such as "something" should be renamed to something more descriptive.
Such names are typically generated automatically
by the browser''s "<i>Create Setter Method</i>" menu function.
'

    "Created: / 03-08-2017 / 14:30:33 / cg"
!

severity
    ^ SeverityInformation

    "Created: / 03-08-2017 / 14:31:12 / cg"
! !

!RBQuestionableIdentifierRule methodsFor:'checking'!

badNames
    ^ #(
            'l' 
            'something'
       ).
!

checkMethod:aContext 
    |badNames skippedSelectors src|

    skippedSelectors := 
        #(
            update:with:from:
        ).
        
    (skippedSelectors includes:aContext selector) ifFalse:[    
        badNames := self badNames.
        src := aContext compiledMethod source.

        badNames do:[:each |
            (src includesString:each) ifTrue:[ 
                super checkMethod:aContext.
                ^ self.
            ].
        ].
    ].

    "Created: / 03-08-2017 / 14:33:02 / cg"
! !

!RBQuestionableIdentifierRule methodsFor:'initialization'!

initialize
    |badNames|

    super initialize.
    
    badNames := self badNames.

    self matcher 
        matches:'`v'
        do:[:variable :answer |
            answer isNil ifTrue:[
                (badNames includes:variable name) ifTrue:[
                    variable
                ] ifFalse:[
                    nil
                ].
            ] ifFalse:[
                answer
            ].
        ]
! !

!RBQuestionableIdentifierRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

