"{ Encoding: utf8 }"

"
 Copyright (C) 2017 Claus Gittinger 

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBParseTreeLintRule subclass:#RBQuestionableMultiAssignRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Parse-tree Rules'
!

!RBQuestionableMultiAssignRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) 2017 Claus Gittinger 

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBQuestionableMultiAssignRule methodsFor:'accessing'!

group
        ^ 'Possible Bugs'
!

name
        ^ 'Possibly a bad assignment (which should be a comparison)'

    "Modified: / 26-08-2017 / 11:27:52 / cg"
!

rationale
        ^ 'Checks for multi-assignments of the form:<br>
<CODE><PRE>    var1 := ( var2 := expr )</PRE></CODE>
where the right side is a parenthized expression.
This could be a typo and actually be meant to be:<br>
<CODE><PRE>    var1 := ( var2 = expr )</PRE></CODE>
or:<br>
<CODE><PRE>    var1 := ( var2 == expr )</PRE></CODE>
Please check the code. 
<br>If it is actually meant to be a multi-assignment,
remove the parentheses.
'

    "Modified: / 26-08-2017 / 11:30:25 / cg"
    "Modified: / 13-03-2019 / 17:00:44 / Claus Gittinger"
! !

!RBQuestionableMultiAssignRule methodsFor:'initialization'!

initialize
        super initialize.
        self matcher 
                matchesAnyOf: #(
                        ' `v1 := ( `v2 := `#n ) '
                )
                do: [ :node :answerSoFar |
                    |rightSide|
                    
                    "/ check if right side is parenthized
                    self assert:(node isAssignment).
                    rightSide := node value.
                    self assert:(rightSide isAssignment).
                    rightSide hasParentheses ifTrue:[
                        node
                    ] ifFalse:[
                        answerSoFar
                    ].    
                ]

    "Modified: / 26-08-2017 / 11:27:18 / cg"
!

testRule
    "code here only to be found by this rule (for testing)"

    |v1 v2|

    v1 := (v2 := nil).

    "
     self new testRule
    "

    "Modified: / 26-08-2017 / 11:24:42 / cg"
! !

!RBQuestionableMultiAssignRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

