"
 Copyright (C) 2019 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBParseTreeLintRule subclass:#RBSendsMessageImplementedInPackageExcludedFromPrerequisitesRule
	instanceVariableNames:'currentContext'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Smalltalk/X'
!

!RBSendsMessageImplementedInPackageExcludedFromPrerequisitesRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) 2019 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBSendsMessageImplementedInPackageExcludedFromPrerequisitesRule class methodsFor:'accessing'!

tagsBasic
    "Return an array of tags (symbols) of given rule. Any rule may
     arbitrary number of tags. Tag values are not predefined/fixed.
     See class documentation for list of common tags"

    ^#( stx )

    "Created: / 01-12-2014 / 11:49:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!RBSendsMessageImplementedInPackageExcludedFromPrerequisitesRule methodsFor:'accessing'!

group
        ^ 'Bugs - Smalltalk/X'

    "Modified: / 01-12-2014 / 11:44:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

name
    ^ 'Refers to a method from a non-prerequisite package'

    "Modified: / 02-12-2014 / 11:04:09 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 03-02-2019 / 14:54:16 / Claus Gittinger"
!

rationale
    ^ 'Check for message sends which might require a method from another package,
which is not included in the prerequisite of the sending method''s package.

If not properly guarded, this may lead to runtime DoesNotUnderstand-errors 
if the referenced method''s package is not loaded. 
<P>You should either update the prerequisites of the package to 
include the other package, or make this method an extension 
(i.e. move it to the other package).
<P>If the dependency is really not wanted, then such a class 
reference should be guarded by some code that checks for package availability and
handle cases when the package is not loaded.'

    "Created: / 07-03-2012 / 17:43:35 / cg"
    "Modified: / 02-12-2014 / 11:04:42 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 03-02-2019 / 14:55:55 / Claus Gittinger"
! !

!RBSendsMessageImplementedInPackageExcludedFromPrerequisitesRule methodsFor:'initialization'!

initialize
    super initialize.
    self matcher 
        matches:'`{:node :context | node isMessage }'
        do:[:node :oldAnswer |
            | newAnswer |

            (newAnswer := oldAnswer) isNil ifTrue:[
                | mthd sel possibleReceivers possibleReceiver
                  packageOfSendingMethod prjDefOfSendingMethod what cls 
                  varName mthdNonMetaclass|

                sel := node selector.
                self assert:currentContext notNil.
                (currentContext notNil) ifTrue:[
                    mthd := currentContext compiledMethod.
                    mthd notNil ifTrue:[
                        packageOfSendingMethod := mthd package.
                        prjDefOfSendingMethod := ProjectDefinition definitionClassForPackage:packageOfSendingMethod.
                        node receiver isGlobal ifTrue:[
                            varName := node receiver name.
                            (mthd mclass allInstVarNames includes:varName) ifFalse:[
                                (mthd mclass theNonMetaclass allClassVarNames includes:varName) ifFalse:[
                                    varName := varName asSymbolIfInterned ? varName.
                                    mthdNonMetaclass := mthd mclass theNonMetaclass.
                                    cls := mthdNonMetaclass privateClassNamed:varName.
                                    cls isNil ifTrue:[
                                        cls := mthdNonMetaclass nameSpace classNamed:varName.
                                        cls isNil ifTrue:[
                                            cls := mthdNonMetaclass topNameSpace classNamed:varName.
                                            cls isNil ifTrue:[
                                                cls := Smalltalk classNamed:varName.
                                            ].    
                                        ].    
                                    ]. 
                                    cls notNil ifTrue:[ 
                                        possibleReceiver := cls class lookupMethodFor:sel.
                                        possibleReceiver notNil ifTrue:[
                                            possibleReceivers := { possibleReceiver }.
                                        ].    
                                    ].    
                                ].
                            ].
                        ].
                        possibleReceivers isNil ifTrue:[
                            possibleReceivers := Smalltalk allImplementorsOf:sel.
                            "/ for now, ignore if are also implemented in any of the stx packages
                            (possibleReceivers contains:[:mthd | mthd package startsWith:'stx:']) ifTrue:[
                                possibleReceivers := #()
                            ].    
                        ].    
                        possibleReceivers do:[:implMthd |
                            |packageOfImplementingMethod prjDefOfImplementingMethod|
                            
                            packageOfImplementingMethod := implMthd package.
                            packageOfImplementingMethod ~= packageOfSendingMethod ifTrue:[
                                prjDefOfImplementingMethod := ProjectDefinition definitionClassForPackage:packageOfImplementingMethod.
                                (prjDefOfSendingMethod allPreRequisites includes:packageOfImplementingMethod) ifFalse:[
                                    what := what ? packageOfImplementingMethod    
                                ].    
                            ].
                        ].
                    ]    
                ].

                (what notNil) ifTrue:[ 
                    newAnswer := node.
                ]
            ].
            newAnswer
        ]

    "Modified: / 31-07-2013 / 17:47:51 / cg"
    "Modified: / 01-12-2014 / 11:52:47 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 03-02-2019 / 16:27:04 / Claus Gittinger"
! !

!RBSendsMessageImplementedInPackageExcludedFromPrerequisitesRule methodsFor:'redefinable utilities'!

resultSelectionIntervalsForSource:aString tree:anRBMethodNode in: aClass do:aBlock
    "evaluate aBlock for all text-intervals 
     (used by the highlighter to underwave affected code fragments)."

    | savedContext |

    savedContext := currentContext.
    [ 
        currentContext := SmalllintContext new.
        currentContext selectedClass: aClass.
        currentContext selector: anRBMethodNode selector.
        super resultSelectionIntervalsForSource:aString tree:anRBMethodNode in: aClass do:aBlock
    ] ensure:[ 
        currentContext := savedContext
    ].

    "Created: / 02-12-2014 / 11:20:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (comment): / 08-02-2017 / 00:06:21 / cg"
! !

!RBSendsMessageImplementedInPackageExcludedFromPrerequisitesRule methodsFor:'running'!

checkMethod: aContext
    
    currentContext := aContext.    
    super checkMethod: aContext.
    currentContext := nil.

    "Created: / 08-07-2013 / 11:35:17 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!RBSendsMessageImplementedInPackageExcludedFromPrerequisitesRule class methodsFor:'documentation'!

version_CVS
    ^ '$Header$'
! !

