"{ Encoding: utf8 }"

"
 Copyright (C) 2014 Claus Gittinger
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBBlockLintRule subclass:#RBSentNotImplementedInVWRule
	instanceVariableNames:''
	classVariableNames:'IgnoredPackages'
	poolDictionaries:''
	category:'Refactory-Lint-VW'
!

!RBSentNotImplementedInVWRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) 2014 Claus Gittinger
"
! !

!RBSentNotImplementedInVWRule class methodsFor:'accessing'!

ignorePackage:aPackage
    IgnoredPackages isNil ifTrue:[
        IgnoredPackages := Set new.
    ].
    IgnoredPackages add:aPackage

    "Created: / 22-05-2020 / 10:20:23 / cg"
!

ignoredPackages
    ^ IgnoredPackages ? #()

    "Created: / 22-05-2020 / 10:19:53 / cg"
!

tagsBasic
    "Return an array of tags (symbols) of given rule. Any rule may
     arbitrary number of tags. Tag values are not predefined/fixed.
     See class documentation for list of common tags"

    ^#( vse )
! !

!RBSentNotImplementedInVWRule methodsFor:'accessing'!

group
        ^ 'VW Portability Issues'

    "Modified: / 14-05-2020 / 10:06:52 / cg"
!

name
        ^ 'Messages known to be not implemented in VW'

    "Modified: / 14-05-2020 / 10:06:47 / cg"
!

rationale
        ^ 'Checks for messages that are sent by a method, but known to be not implemented in (base) VW Smalltalk.
<P>
This is a special rule useful only in projects which contain code which must be 
portable to VW Smalltalk.
Currently, this is the expeccoBridge, for which this is relevant.
Regular users should leave this rule disabled.'

    "Modified: / 14-05-2020 / 10:06:40 / cg"
!

severity
        ^ #warning
! !

!RBSentNotImplementedInVWRule methodsFor:'running'!

checkMethod: aContext 
    |selectedClass messages messagesKnownToMakeProblems checkedSelectors|

    selectedClass := aContext selectedClass.
    (self class ignoredPackages includes:selectedClass package) ifTrue:[
        ^ self
    ].

    checkedSelectors := 
        #(
            #'?'
            #'nextPutLine:'
            #'copyTo:'
            #'copyLast:'
            #'copyFrom:'
            #'answer:do:'
            #'ignoreIn:'
            #'loop'

            #'conform:'
            "/ not in typical install:
            #'isEmptyOrNil'
            #'isNilOrEmpty'
            #'notEmptyOrNil'
            #'notNilOrEmpty'
            #'second'
            #'third'
            #'isArray'
        ).

    messages := aContext messages.
    messagesKnownToMakeProblems := messages select:[:selector | checkedSelectors includes:selector ].

    messagesKnownToMakeProblems notEmptyOrNil ifTrue:[
        messagesKnownToMakeProblems do:[:each |
            result addSearchString: each.
        ].
        result 
            addClass: selectedClass
            selector: aContext selector 
    ]

    "Modified: / 24-05-2020 / 20:53:38 / cg"
! !

!RBSentNotImplementedInVWRule class methodsFor:'documentation'!

version_CVS
    ^ '$Header$'

! !

