"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011-2013 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011-2013 Claus Gittinger (porting, Smalltalk/X rules & tweaks)

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBBlockLintRule subclass:#RBSentNotImplementedRule
	instanceVariableNames:'badMessagesPerMethod'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Block Rules'
!

!RBSentNotImplementedRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011-2013 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011-2013 Claus Gittinger (porting, Smalltalk/X rules & tweaks)

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
!

documentation
"
    documentation to be added.

    [author:]
        cg

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!RBSentNotImplementedRule methodsFor:'accessing'!

group
	^ 'Bugs'
!

name
	^ 'Messages sent but not implemented'
!

rationale
    |t sel badMessages methodsPerBadSelector|

    t := self rationaleText.
    badMessages := IdentitySet new.
    methodsPerBadSelector := Dictionary new.

    badMessagesPerMethod notNil ifTrue:[
        badMessagesPerMethod keysAndValuesDo:[:classNamePair :messagesPerMethod |
            messagesPerMethod do:[:eachBadMessage |
                (methodsPerBadSelector at:eachBadMessage ifAbsentPut:[Set new]) add:classNamePair
            ].
            badMessages addAll:messagesPerMethod
        ].
    ].

    ^ t , (String streamContents:[:ts |
        badMessages notEmpty ifTrue:[
            ts nextPutAll:'<p>Questionable selectors are:<blockquote>'.
            badMessages asNewOrderedCollection sort do:[:eachBadSelector |
                |badMethods classNamePair className selector|

                ts nextPutAll:'<code>'; nextPutAll:eachBadSelector; nextPutAll:'</code>'.
                (badMethods := methodsPerBadSelector at:eachBadSelector ifAbsent:nil) notNil ifTrue:[
                    classNamePair := badMethods first.
                    className := classNamePair first.
                    selector := classNamePair second.
                    ts nextPutAll:' (in: <A INFO="Click to browse this method" ';
                       nextPutAll:'ACTION="doit: SystemBrowser default openInClass:',className,' selector:#''',selector,'''">';
                       nextPutAll:className;
                       nextPutAll:' � ';
                       nextPutAll:selector;
                       nextPutAll:'</A> '.

                    badMethods size > 1 ifTrue:[
                        ts nextPutAll:' and ',(badMethods size-1)printString,' others'.
                    ].
                    ts nextPutAll:')'.
                ].
                ts nextPutAll:'<br>'
            ].
            ts nextPutAll:'</blockquote>'.
            (badMessages size == 1
              and:[ (sel := badMessages first) isUppercaseFirst
              and:[ (Smalltalk classNamed:sel) notNil ]]
            ) ifTrue:[
                ts nextPutAll:'Here it looks like a period is missing, before the selector.'
            ].
        ] ifFalse:[
            ts nextPutAll:'<P>And, check for a missing period before the selector.'.
        ].
    ]).

    "Modified: / 17-07-2017 / 13:09:00 / cg"
    "Modified: / 29-01-2018 / 11:24:57 / mawalch"
!

rationaleText

    ^ 'Checks for messages that are sent by a method, but no class in the system implements such a message.
Further checks if messages sent to "<I>self</I>" or "<I>super</I>" exist in the hierarchy, since these can be statically typed.
Reported methods will certainly cause a <I>doesNotUnderstand:</I>-message when they are executed.
<P>
Sometimes, this is reported for messages to methods which are loaded as package extensions,
or to classes which are not yet loaded.
<br>
If this is the case, make sure that the code is prepared for those being absent.
<P>
Also, check if the receiver class does not redefine the <code>doesNotUnderstand:</code>-method,
which may interpret such missing messages arbitrarily.'.

    "Created: / 17-07-2017 / 12:33:32 / cg"
!

rationaleWithAnchorForClass:aClass
    "called by the smallint browser, when a linted method is selected.
     Fallback here: same as rationaleWithAnchor
     Can be redefined to return method-specific rationale text in the smallLint viewer"

    |t sel badMessages| 

    t := self rationaleText.
    badMessages := IdentitySet new.
    badMessagesPerMethod keysAndValuesDo:[:classNamePair :messagesPerMethod |
        classNamePair first = aClass name ifTrue:[
            badMessages addAll:messagesPerMethod
        ].    
    ].
    badMessages notEmpty ifTrue:[
        t := t , '<p>Questionable selectors in this class are:<blockquote>'.
        badMessages asNewOrderedCollection sort do:[:eachBadSelector |
            t := t , ('<code>',eachBadSelector,'</code><br>').
        ].
        t := t,'</blockquote>'.
        (badMessages size == 1
          and:[ (sel := badMessages first) isUppercaseFirst
          and:[ (Smalltalk classNamed:sel) notNil ]]
        ) ifTrue:[
            t := t,'Here it looks like a period is missing, before the selector.'
        ].    
    ] ifFalse:[    
        t := t,'<P>And, check for a missing period before the selector.'.
    ].
    ^ t

    "Created: / 17-07-2017 / 12:43:02 / cg"
!

rationaleWithAnchorForMethod:aMethod
    "called by the smallint browser, when a linted method is selected.
     Fallback here: same as rationaleWithAnchor
     Can be redefined to return method-specific rationale text in the smallLint viewer"

    |t sel badMessages| 

    t := self rationaleText.
    (badMessages := badMessagesPerMethod at:{aMethod mclass name . aMethod selector} ifAbsent:nil) notEmptyOrNil ifTrue:[
        t := t , '<p>Questionable selectors in this method are:<blockquote>'.
        badMessages asNewOrderedCollection sort do:[:eachBadSelector |
            t := t , ('<code>',eachBadSelector,'</code><br>').
        ].
        t := t,'</blockquote>'.
        (badMessages size == 1
          and:[ (sel := badMessages first) isUppercaseFirst
          and:[ (Smalltalk classNamed:sel) notNil ]]
        ) ifTrue:[
            t := t,'Here it looks like a period is missing, before the selector.'
        ].    
    ] ifFalse:[    
        t := t,'<P>And, check for a missing period before the selector.'.
    ].
    ^ t

    "Created: / 17-07-2017 / 12:33:19 / cg"
!

severity
        ^ SeverityError
! !

!RBSentNotImplementedRule methodsFor:'running'!

checkMethod: aContext 
        |addBlock selectedClass selector messages foreignSelectors|

        "/ cg: huh - what is checked here.
        "/      (aContext implements: each) not
        "/ checks if message is implemented inside the context,
        "/ but I think, it should check if it is implemented ANYWHERE.

"/        message := aContext messages 
"/            detect: [ :each | (aContext implements: each) not ]
"/            ifNone: [ 
"/                aContext superMessages 
"/                    detect:[ :each | 
"/                        aContext selectedClass superclass isNil or: [ (aContext selectedClass superclass canUnderstand: each) not ] ]
"/                    ifNone:[ 
"/                        aContext selfMessages 
"/                            detect: [ :each | (aContext selectedClass canUnderstand: each) not ]
"/                            ifNone: [ nil ] 
"/                    ] 
"/            ].

    selectedClass := aContext selectedClass.
    selector := aContext selector.
    
    addBlock := 
        [:messageSelector|
            |mthd anno|

            foreignSelectors isNil ifTrue:[
                mthd := aContext compiledMethod.
                mthd notNil ifTrue:[
                    mthd := [mthd method] on:Error do:[mthd].
                    foreignSelectors := #().
                    (anno := mthd annotationAt:#foreignSelectors:) notNil ifTrue:[
                        foreignSelectors := anno arguments at:1.
                    ].
                    (anno := mthd annotationAt:#foreignSelector:) notNil ifTrue:[
                        foreignSelectors := { anno arguments at:1 }.
                    ].
                ].
            ].
            (foreignSelectors includes:messageSelector) ifFalse:[
                result 
                    addSearchString: messageSelector;
                    addClass: selectedClass
                    selector: selector.
                badMessagesPerMethod isNil ifTrue:[
                    badMessagesPerMethod := Dictionary new.
                ].
                (badMessagesPerMethod at:{selectedClass name . selector} ifAbsentPut:[Set new]) add:messageSelector.
            ].
        ].

    aContext selfMessages do:[:each|
        (selectedClass canUnderstand: each) ifFalse:[
            each ~~ #== ifTrue:[    "/ #== is a Smalltalk builtin.
                addBlock value:each.
            ].
        ].
    ].
    aContext superMessages do:[:each|
        (selectedClass superclass notNil and:[selectedClass superclass canUnderstand: each]) ifFalse:[
            addBlock value:each.
        ].
    ].

    messages := aContext messages copy.

    aContext parseTree allMessageNodesDo:[:eachMessageNode| 
        |cls selector|

        "finding private classes with the same name as a top-level class does not work yet!!"
        (eachMessageNode receiver isGlobal 
         and:[(cls := selectedClass topNameSpace classNamed:eachMessageNode receiver name) notNil
              or:[(cls := Smalltalk classNamed:eachMessageNode receiver name) notNil and:[cls isNameSpace not]]
         ]) ifTrue:[
            selector :=  eachMessageNode selector.
            (cls respondsTo:selector) ifFalse:[
                addBlock value:selector.
            ].
            messages remove:selector ifAbsent:[].
        ].
    ].

    messages do:[:each|
        (Smalltalk someImplementorOf:each) isNil ifTrue:[
            addBlock value:each.
        ].
    ].

    "Modified: / 26-09-2017 / 12:10:28 / stefan"
    "Modified (format): / 24-05-2020 / 11:27:51 / cg"
! !

!RBSentNotImplementedRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

