"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011-2013 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011-2013 Claus Gittinger (porting, Smalltalk/X rules & tweaks)

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBBlockLintRule subclass:#RBSentObsoleteMessageRule
	instanceVariableNames:''
	classVariableNames:'ObsoleteMethods ObsoleteMethodsSelectors'
	poolDictionaries:''
	category:'Refactory-Lint-Smalltalk/X'
!

!RBSentObsoleteMessageRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011-2013 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011-2013 Claus Gittinger (porting, Smalltalk/X rules & tweaks)

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBSentObsoleteMessageRule class methodsFor:'utilities'!

allObsoleteMethodsSelectors
    |obsolete|

    (obsolete := ObsoleteMethodsSelectors) isNil ifTrue:[
        obsolete := self getAllObsoleteMethodsSelectors.
    ].
    ^ obsolete

    "
     self allObsoleteMethodsSelectors
    "

    "Created: / 14-07-2017 / 10:49:58 / cg"
!

getAllObsoleteMethodsSelectors
    |obsolete|
    
    Smalltalk removeDependent:self.
    obsolete := Set new.
    Smalltalk allMethodsWithSelectorDo:[:m :sel | m isObsolete ifTrue:[obsolete add:sel]].
    ObsoleteMethodsSelectors := obsolete.
    Smalltalk addDependent:self.
    ^ obsolete

    "
     self getAllObsoleteMethodsSelectors
    "

    "Created: / 14-07-2017 / 10:55:44 / cg"
!

implementorsOf:aSelector detect:aBlock
    |atLeastOneMethodFound|

    atLeastOneMethodFound := false.
    Smalltalk allImplementorsOf:aSelector do:[:eachMethod| 
        (aBlock value:eachMethod) ifTrue:[^ eachMethod].
        atLeastOneMethodFound := true.
    ].
    ^ atLeastOneMethodFound.

    "Created: / 31-03-2017 / 14:23:42 / stefan"
!

update:what with:aParameter from:changedObject
    changedObject == Smalltalk ifTrue:[
        ObsoleteMethods := nil.
        ^ self
    ].    
    ^ super update:what with:aParameter from:changedObject.

    "Created: / 14-07-2017 / 10:41:10 / cg"
! !

!RBSentObsoleteMessageRule methodsFor:'accessing'!

group
    ^ 'Miscellaneous'

    "Modified: / 31-03-2017 / 14:07:17 / stefan"
!

name
    ^ 'Messages sent but marked as obsolete'

    "Modified (format): / 31-03-2017 / 14:08:28 / stefan"
!

rationale
        ^ 'Checks for obsolete messages that are sent by a method.'

    "Modified: / 31-03-2017 / 14:28:51 / stefan"
! !

!RBSentObsoleteMessageRule methodsFor:'running'!

checkMethod: aContext 
    |cls selector addBlock selectedClass method obsoleteMethodSelectors|

    aContext compiledMethod isObsolete ifTrue:[
        "do not check methods that are obsolete themself"
        ^ self.
    ].

    cls := aContext selectedClass.
    selector := aContext selector.

    addBlock := 
        [:badMessageSelector|
            result 
                addSearchString: badMessageSelector;
                addClass: cls
                selector: selector.
        ].

    obsoleteMethodSelectors := self class allObsoleteMethodsSelectors.
    
    selectedClass := cls.

    aContext selfMessages do:[:eachSelector|
        (obsoleteMethodSelectors includes:eachSelector) ifTrue:[
            method := selectedClass lookupMethodFor:eachSelector.
            (method notNil and:[method isObsolete]) ifTrue:[
                addBlock value:eachSelector.
            ].
        ].
    ].
    selectedClass superclass notNil ifTrue:[
        aContext superMessages do:[:eachSelector|
            (obsoleteMethodSelectors includes:eachSelector) ifTrue:[
                method := selectedClass superclass lookupMethodFor:eachSelector.
                (method notNil and:[method isObsolete]) ifTrue:[
                    addBlock value:eachSelector.
                ].
            ].
        ].
    ].

    aContext messages do:[:eachSelector|
        (obsoleteMethodSelectors includes:eachSelector) ifTrue:[
            method := self class implementorsOf:eachSelector detect:[:eachMethod| eachMethod isObsolete not].
            method == true ifTrue:[
                addBlock value:eachSelector.
            ].
        ].
    ].

    "Modified: / 31-03-2017 / 14:44:39 / stefan"
    "Modified: / 14-07-2017 / 10:57:23 / cg"
! !

!RBSentObsoleteMessageRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

