"{ Encoding: utf8 }"

"
 Copyright (C) 2019 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBParseTreeLintRule subclass:#RBStringManipulationRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Parse-tree Rules'
!

!RBStringManipulationRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) 2019 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBStringManipulationRule methodsFor:'accessing'!

group
	^ 'Miscellaneous'
!

name
        ^ 'String manipulations done obfuscated, overly complex or suboptimal '

    "Modified: / 13-03-2019 / 18:05:19 / Claus Gittinger"
!

rationale
        ^ 'Check for string manipulation messages which can be done
more performant, shorter or simply more readable. 
<p>
For example, <code><pre>   expr emphasizeAllWith:#bold</pre></code>
should be rewritten to:<code><pre>   expr allBold</pre></code>
and: <code><pre>   expr emphasizeAllWith:(#color -> colorExpr)</pre></code>
is easier to read when written:<code><pre>   expr withColor: colorExpr</pre></code>
<p>
Both String and Text instances understand #allBold and #withColor:,
so "asText" can be removed in <code><pre>   expr asText allBold</pre></code>
for a shorter <code><pre>   expr allBold</pre></code>'

    "Modified: / 17-02-2017 / 08:40:17 / cg"
    "Modified: / 13-03-2019 / 20:59:59 / Claus Gittinger"
!

severity
        ^ SeverityInformation

    "Created: / 13-03-2019 / 18:11:12 / Claus Gittinger"
! !

!RBStringManipulationRule methodsFor:'checking'!

checkMethod: aContext
    "skip check for methods which implement the tested code"

    (aContext selectedClass == CharacterArray) ifTrue:[^ self].
    ^ super checkMethod:aContext.

    "Created: / 13-03-2019 / 21:07:56 / Claus Gittinger"
! !

!RBStringManipulationRule methodsFor:'initialization'!

initialize
    super initialize.
    self matcher
        matchesAnyOf:#(
                ' `@expr emphasizeAllWith: #bold'
                ' `@expr emphasizeAllWith:( #color -> `@colorExpr )'
                ' `@expr asText allBold'
                ' `@expr asText allItalic'
                ' `@expr asText allUnderlined'
                ' `@expr asText withColor: `@colorExpr'
                ' (String new:`@expr1) atAllPut:`@valExpr'
                "/ ' `@expr1 from:1 to:(`@expr1 size) put: `@valExpr'
        )        
        do:[:node :answer | node].

    "Modified: / 20-08-2013 / 09:13:27 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 26-03-2019 / 11:59:29 / Claus Gittinger"
! !

!RBStringManipulationRule class methodsFor:'documentation'!

version_CVS
    ^ '$Header$'
! !

