"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011-2013 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011-2013 Claus Gittinger (porting, Smalltalk/X rules & tweaks)

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBBlockLintRule subclass:#RBSubclassResponsibilityNotDefinedRule
	instanceVariableNames:'missingInClassPerSelector'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Block Rules'
!

!RBSubclassResponsibilityNotDefinedRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011-2013 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011-2013 Claus Gittinger (porting, Smalltalk/X rules & tweaks)

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
!

documentation
"
    This rules identifies when several methods in a set of subclasses 
    are defined but not in the superclass. 
    It identifies that an abstract method is missing in the superclass.

"
! !

!RBSubclassResponsibilityNotDefinedRule methodsFor:'accessing'!

group
	^ 'Bugs'
!

name
        ^ 'Subclass responsibility not defined in subclass(es)'

    "Modified: / 18-05-2018 / 12:34:31 / Stefan Vogel"
!

rationale
        ^ 'Checks that all subclassResponsibility methods are defined in all leaf sub-classes.
(I.e. looks from a superclass down to its subclasses)'

    "Modified: / 07-03-2012 / 21:10:37 / cg"
!

rationaleWithAnchorForClass:aClass
    "called by the smallint browser, when a linted class is selected."

    |s missingInstSelectors missingClassSelectors missingHere missingInSubclasses|

    s := String 
        streamContents:[:s |
            s nextPutAll:self rationaleWithAnchor.
            result notNil ifTrue:[
                missingInstSelectors := result classSelectors at:aClass theNonMetaclass name ifAbsent:nil.
                missingClassSelectors := result metaClassSelectors at:aClass theNonMetaclass name ifAbsent:nil.
                missingInstSelectors notEmptyOrNil ifTrue:[
                    missingInstSelectors := missingInstSelectors asNewOrderedCollection sort.

                    missingHere := missingInstSelectors reject:[:sel | aClass theNonMetaclass includesSelector:sel].
                    missingInSubclasses := missingInstSelectors select:[:sel | aClass theNonMetaclass includesSelector:sel].

                    missingHere notEmpty ifTrue:[
                        "/ generate HTML to create the method on click
                        s nextPutAll: '<p>Missing instance methods (click to create & browse):<br>'.
                        missingHere
                            do:[:each |
                                s nextPutAll:'<A INFO="Click to create & browse this method"'.
                                s nextPutAll:(' ACTION="doit: self createMissingMethodAndBrowseIn:%1 theNonMetaclass selector:#%2">'
                                                        bindWith:aClass theNonMetaclass name with:each).
                                s nextPutAll:each.
                                s nextPutAll:'</A>'
                            ]
                            separatedBy:[
                                s nextPutAll:', '
                            ]
                    ].    
                    missingInSubclasses notEmpty ifTrue:[
                        "/ generate HTML to show missing names
                        s nextPutAll: '<p>Missing instance method redefinitions (in some subclass):<br>'.
                        s nextPutAll:(missingInSubclasses asStringCollection asStringWith:', ').
                    ].    
                ]. 

                missingClassSelectors notEmptyOrNil ifTrue:[
                    missingClassSelectors := missingClassSelectors asNewOrderedCollection sort.

                    missingHere := missingClassSelectors reject:[:sel | aClass theNonMetaclass includesSelector:sel].
                    missingInSubclasses := missingClassSelectors select:[:sel | aClass theNonMetaclass includesSelector:sel].

                    missingHere notEmpty ifTrue:[
                        "/ generate HTML to create the method on click
                        s nextPutAll: '<p>Missing class methods: (click to create & browse)<br>'.
                        missingHere
                            do:[:each |
                                s nextPutAll:'<A INFO="Click to create & browse this method"'.
                                s nextPutAll:(' ACTION="doit: self createMissingMethodAndBrowseIn:%1 selector:#%2">'
                                                        bindWith:aClass theMetaclass name with:each).
                                s nextPutAll:each.
                                s nextPutAll:'</A>'
                            ]
                            separatedBy:[
                                s nextPutAll:', '
                            ]
                    ].
                    missingInSubclasses notEmpty ifTrue:[
                        "/ generate HTML to create the method on click
                        s nextPutAll: '<p>Missing class method redefinitions: (in some subclass)<br>'.
                        s nextPutAll:(missingInSubclasses asStringCollection asStringWith:', ').
                    ].
                ].
            ].
        ].
    ^ s


    "Created: / 30-03-2017 / 09:45:55 / cg"
    "Modified: / 30-03-2017 / 11:15:01 / cg"
!

severity
        ^ SeverityError
! !

!RBSubclassResponsibilityNotDefinedRule methodsFor:'rationale callbacks'!

createMissingMethodAndBrowseIn:aClass selector:aSelector
    "callback when a doit-anchor is clicked in the rationale text"

    |spec code implementor|

    implementor := aClass whichClassIncludesSelector:aSelector.
    spec := Parser methodSpecificationForSelector:aSelector.

    code := spec,'
    "this method must be redefined as stated in a superclass"

    self shouldImplement.
'.
    Compiler compile:code forClass:aClass inCategory:(implementor category).
    Smalltalk browseInClass:aClass selector:aSelector

    "Modified: / 16-11-2018 / 09:17:58 / Claus Gittinger"
! !

!RBSubclassResponsibilityNotDefinedRule methodsFor:'running'!

checkClass:aContext 
    |theClass allMethodsWithSubclassResponsibility subclasses|

    theClass := aContext selectedClass.
    subclasses := theClass withAllSubclasses.

    allMethodsWithSubclassResponsibility := 
        theClass methodDictionary values 
            select:[:each | each isSubclassResponsibility].

    allMethodsWithSubclassResponsibility do:[:each |
        |mthdSelector|

        mthdSelector := each selector.
        subclasses do:[:class |
            (class subclasses isEmpty "so its a leaf class"
                and:[ class theNonMetaclass isAbstract not
                and:[ (class whichClassIncludesSelector:mthdSelector) == theClass ]])
                ifTrue:[
                    "/ there is no such method in the subclass...
                    "/ ... so the following is probably useless.
                    self 
                        addClass:class
                        selector:mthdSelector                        
                        to:result.
                    "/ so we add the superclass.
                    self 
                        addClass:theClass
                        selector:mthdSelector                        
                        to:result.
                    "/ and remember which class should have that method,
                    "/ so we can show it in the rationale string.
                    missingInClassPerSelector isNil ifTrue:[
                        missingInClassPerSelector := Dictionary new.
                    ].
                    (missingInClassPerSelector at:mthdSelector ifAbsentPut:[Set new]) add:class.

                ]
        ]
    ]

    "Modified (format): / 22-01-2013 / 18:42:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 16-07-2017 / 11:31:42 / cg"
! !

!RBSubclassResponsibilityNotDefinedRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

