"{ Encoding: utf8 }"

"
 Copyright (C) 2017 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBBlockLintRule subclass:#RBSubclassResponsibilityNotReimplementedRule
	instanceVariableNames:'allSelectorsWithSubclassResponsibility
		allSelectorsWithoutSubclassResponsibility'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Block Rules'
!

!RBSubclassResponsibilityNotReimplementedRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) 2017 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
!

documentation
"
    This rules identifies methods which should have been redefined as concrete
    in this class or a superclass. I.e. it identifies messages which could lead
    to a subclassResponsibility error at runtime.
"
! !

!RBSubclassResponsibilityNotReimplementedRule methodsFor:'accessing'!

group
	^ 'Bugs'
!

name
    ^ 'Method marked as subclassResponsibility in superclass but not redefined as concrete'

    "Modified: / 10-02-2017 / 19:03:24 / cg"
!

rationale
    ^ 'Checks that all methods which are marked as subclassResponsibility in a superclass
are reimplemented as concrete methods and understood by instances of this class.
(I.e. looks from a subclass up to its superclasses)
<p>This rule ignores classes marked as being abstract.'

    "Modified: / 10-02-2017 / 19:03:42 / cg"
!

rationaleWithAnchorForClass:aClass
    "called by the smallint browser, when a linted class is selected."

    |s missingInstSelectors missingClassSelectors|

    s := String 
        streamContents:[:s |
            s nextPutAll:self rationaleWithAnchor.
            result notNil ifTrue:[
                missingInstSelectors := result classSelectors at:aClass theNonMetaclass name ifAbsent:nil.
                missingClassSelectors := result metaClassSelectors at:aClass theNonMetaclass name ifAbsent:nil.
                missingInstSelectors notEmptyOrNil ifTrue:[
                    missingInstSelectors asNewOrderedCollection sort.
                    s nextPutAll: '<p>Missing instance method redefinitions (click to create & browse):<br>'.
                    missingInstSelectors 
                        do:[:each |
                            s nextPutAll:'<A INFO="Click to create & browse this method"'.
                            s nextPutAll:(' ACTION="doit: self createMissingMethodAndBrowseIn:%1 theNonMetaclass selector:#%2">'
                                                    bindWith:aClass theNonMetaclass name with:each).
                            s nextPutAll:each.
                            s nextPutAll:'</A>'
                        ]
                        separatedBy:[
                            s nextPutAll:', '
                        ]
                ].    
                missingClassSelectors notEmptyOrNil ifTrue:[
                    missingClassSelectors asNewOrderedCollection sort.
                    s nextPutAll: '<p>Missing class method redefinitions: (click to create & browse)<br>'.
                    missingClassSelectors
                        do:[:each |
                            s nextPutAll:'<A INFO="Click to create & browse this method"'.
                            s nextPutAll:(' ACTION="doit: self createMissingMethodAndBrowseIn:%1 theMetaclass selector:#%2">'
                                                    bindWith:aClass theNonMetaclass name with:each).
                            s nextPutAll:each.
                            s nextPutAll:'</A>'
                        ]
                        separatedBy:[
                            s nextPutAll:', '
                        ]
                ].
            ].
        ].
    ^ s


    "Created: / 30-03-2017 / 09:45:55 / cg"
    "Modified: / 30-03-2017 / 11:15:01 / cg"
!

resultClass
    ^ SelectorEnvironmentWithInheritedMethods

    "Created: / 10-02-2017 / 13:20:14 / cg"
!

severity
        ^ SeverityError
! !

!RBSubclassResponsibilityNotReimplementedRule methodsFor:'redefined result access'!

failedMethodsInAnyOf: classes meta: meta
    | klasses |

    klasses := meta 
                ifTrue:[classes collect:[:cls|cls theMetaclass]]
                ifFalse:[classes collect:[:cls|cls theNonMetaclass]].

    ^ self failedMethods 
        select:[:mth| 
            klasses contains:[:someClass |
                someClass includesBehavior: mth mclass
            ]
        ].

    "Created: / 10-02-2017 / 12:59:57 / cg"
! !

!RBSubclassResponsibilityNotReimplementedRule methodsFor:'running'!

checkClass:aContext 
    |theClass|

    theClass := aContext selectedClass.
    theClass theNonMetaClass isAbstract ifTrue:[    
        ^ self
    ].

    allSelectorsWithSubclassResponsibility isNil ifTrue:[
        allSelectorsWithSubclassResponsibility := Set new.
    ].
    allSelectorsWithoutSubclassResponsibility isNil ifTrue:[
        allSelectorsWithoutSubclassResponsibility := Set new.
    ].

    theClass selectorsAndMethodsDo:[:sel :mthd | 
        (mthd isSubclassResponsibility) ifTrue:[
            allSelectorsWithSubclassResponsibility add:sel
        ] ifFalse:[
            (allSelectorsWithSubclassResponsibility includes:sel) ifFalse:[
                allSelectorsWithoutSubclassResponsibility add:sel
            ]
        ]
    ].
    theClass allSuperclassesDo:[:eachSuper |
        eachSuper selectorsAndMethodsDo:[:sel :mthd | 
            (mthd isSubclassResponsibility) ifTrue:[
                (allSelectorsWithoutSubclassResponsibility includes:sel) ifFalse:[
                    result 
                        addClass:theClass
                        selector:sel
                ].    
                allSelectorsWithoutSubclassResponsibility add:sel
            ] ifFalse:[
                allSelectorsWithoutSubclassResponsibility add:sel
            ]
        ].
    ].

    "Modified (format): / 22-01-2013 / 18:42:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 16-07-2017 / 11:32:01 / cg"
    "Modified: / 28-11-2017 / 13:38:22 / stefan"
! !

!RBSubclassResponsibilityNotReimplementedRule methodsFor:'testing'!

isMultiClassSelectorScope
    "true if I am a selector scope containing methods for multiple
     classes. That is, even though I check a particular class,
     my scope will include methods from other classes.
     For example, the subclassResponsibilityNotRedefined rule
     will provide as methods the inherited methods from superclasses,
     not the missing ones (which do not exist, so how could it).
     This information is needed by the lint-result browser's methodList, 
     to show inherited methods for this selected class"

    ^ true

    "Created: / 10-02-2017 / 12:57:05 / cg"
! !

!RBSubclassResponsibilityNotReimplementedRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

