"{ Encoding: utf8 }"

"
 Copyright (C) 2020 Claus Gittinger
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBParseTreeLintRule subclass:#RBSyntaxUnportableToVWCheckRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-VW'
!

!RBSyntaxUnportableToVWCheckRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) 2020 Claus Gittinger
"
! !

!RBSyntaxUnportableToVWCheckRule class methodsFor:'accessing'!

tagsBasic
    "Return an array of tags (symbols) of given rule. Any rule may
     arbitrary number of tags. Tag values are not predefined/fixed.
     See class documentation for list of common tags"

    ^#( vw )

    "Modified: / 07-05-2020 / 12:37:43 / cg"
! !

!RBSyntaxUnportableToVWCheckRule methodsFor:'accessing'!

group
        ^ 'VW Portability Issues'

    "Modified: / 07-05-2020 / 12:37:56 / cg"
!

name
    ^ 'Syntactic construct not supported by VW'

    "Modified: / 10-05-2020 / 10:10:28 / cg"
!

rationale
        ^ 'Checks for syntactic constructs which are not available in VisualWorks Smalltalk.
<P>
These are (currently):
<ul>
<li>EOL or Token comments
<li>Brace array constructor "{ ... }"
</ul>
<p>
This is only of interest if you plan to port the code to a VisualWorks system.'

    "Modified: / 17-05-2017 / 16:58:24 / mawalch"
    "Modified: / 30-05-2020 / 11:19:50 / cg"
! !

!RBSyntaxUnportableToVWCheckRule methodsFor:'checking'!

checkMethod:aContext
    | cls source |

    super checkMethod:aContext.

    aContext isNil ifTrue:[
        "/ My guess: happens when two rules are executed in parallel (seems to no longer happen)
        self breakPoint:#cg.      
        ^ self.
    ].   

    cls := aContext selectedClass.

    source := aContext sourceCode.
    "/ a quick rejector

    ((source includesString:'"/') or:[ (source includesString:'"<<') ]) ifFalse:[^ self].
    RBContainsSmalltalkXEOLCommentRule eolAndTokenCommentsIn: source do:[:each | 
        self 
            addClass: cls 
            selector: aContext selector 
            to: result.
        ^ self.
    ].

    "Created: / 15-05-2020 / 13:58:26 / cg"
! !

!RBSyntaxUnportableToVWCheckRule methodsFor:'initialization'!

initialize
    super initialize.
    self matcher 
            matchesAnyOf: #(
                    '{ `@.arrayExpressionStatements }'      "/ does not exist at all in VW
            )
            do: [ :node :answer | node ]

    "Modified: / 07-05-2020 / 12:38:13 / cg"
! !

!RBSyntaxUnportableToVWCheckRule class methodsFor:'documentation'!

version_CVS
    ^ '$Header$'
! !

