"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011-2013 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011-2013 Claus Gittinger (porting, Smalltalk/X rules & tweaks)

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBParseTreeLintRule subclass:#RBToDoRule
	instanceVariableNames:'nodesSendingAt'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Parse-tree Rules'
!

!RBToDoRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011-2013 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011-2013 Claus Gittinger (porting, Smalltalk/X rules & tweaks)

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBToDoRule methodsFor:'accessing'!

group
        ^ 'Intention Revealing'
!

name
        ^ 'Uses "to:do:" instead of "do:", "doWithIndex:", "with:do:" or "timesRepeat:"'

    "Modified: / 06-02-2017 / 16:14:34 / cg"
!

rationale
        ^ 'Checks for uses of "<code>to:do:</code>" 
when a "<code>do:</code>", "<code>with:do:</code>", "<code>do:withIndex:</code>" or "<code>timesRepeat:</code>" 
should be used.'

    "Modified: / 06-02-2017 / 16:14:55 / cg"
!

sourceIntervalForNode:aMatchedNode inTree:aParseTree source:fullSource
    "redefined, to only return the interval for the first at: message,
     not the whole expression node 
     (which may spawn many many lines if the block is large)."

    nodesSendingAt notEmptyOrNil ifTrue:[
        nodesSendingAt do:[:eachSendingNode |
            (eachSendingNode isChildOf:aMatchedNode) ifTrue:[
                ^ eachSendingNode start to:eachSendingNode stop.
            ].
        ].
    ].
    ^ super sourceIntervalForNode:aMatchedNode inTree:aParseTree source:fullSource

    "Created: / 19-02-2017 / 13:41:17 / cg"
! !

!RBToDoRule methodsFor:'initialization'!

initialize
    super initialize.
    self matcher 
        matches:'1 to: ``@object size do: [:`each | | `@temps | `@.Statements]'
        do:[:node :answer :owner| 
            answer isNil ifTrue:[
                |varName variableMatcher|

                varName := node arguments first receiver.
                (owner isNil
                or:[ ((varName ~= 'self') 
                     or:[ owner currentSearchContext selectedClass realClass canUnderstand: #do: ])]) ifTrue:[
                    "/ check if '@object at:' is sent inside the statements
                    nodesSendingAt isNil ifTrue:[ nodesSendingAt := OrderedCollection new ].
                    variableMatcher := ParseTreeSearcher new.
                    variableMatcher 
                        matchesTree:varName
                        do:[:nod :ans | 
                            |sel|
                            
                            (nod parent isMessage 
                              and:[ ((sel := nod parent selector) = #at:)
                                    or:[ sel = #at:ifAbsent:] ]) ifTrue:[
                                nodesSendingAt add:nod parent.
                                true
                            ] ifFalse:[    
                                ans
                            ]
                        ].

                    (variableMatcher executeTree:node arguments last body initialAnswer:false) ifTrue:[
                        node
                    ] ifFalse:[ 
                        nil 
                    ]
                ] ifFalse:[ 
                    nil 
                ]
            ] ifFalse:[ 
                answer 
            ]
        ]

    "Modified (format): / 19-02-2017 / 13:45:49 / cg"
! !

!RBToDoRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

