"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011-2013 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011-2013 Claus Gittinger (porting, Smalltalk/X rules & tweaks)

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBParseTreeLintRule subclass:#RBUsesAddRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Parse-tree Rules'
!

!RBUsesAddRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011-2013 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011-2013 Claus Gittinger (porting, Smalltalk/X rules & tweaks)

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBUsesAddRule methodsFor:'accessing'!

group
        ^ 'Possible Bugs'
!

name
        ^ 'Uses the result of an "add:" message'

    "Modified: / 01-03-2012 / 11:27:22 / cg"
!

rationale
        ^ 'Check for possible uses of the result returned by an "<code>add:</code>" or "<code>addAll:</code>" messages. 
By convention, these messages return their argument(s), <b>not</b> the receiver. 
So the value here may be the argument, not the receiver collection.
<br>
If the receiver is what you want, add a "<code>; yourself</code>"-cascade;
if not, assign the argument to a temporary and use that in followup code.'

    "Modified: / 01-03-2012 / 12:17:42 / cg"
!

severity
    "redefine to return one of:
        SeverityInformation
        SeverityError
        SeverityWarning (no need to redefine - except for documentation)
    "

    ^ SeverityInformation

    "Created: / 14-03-2011 / 14:11:02 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!RBUsesAddRule methodsFor:'initialization'!

checkMethod: aContext
    |mthd checkedSelectors|

    checkedSelectors := #( add: addFirst: addLast: addAll: addAllLast:).

    (checkedSelectors includes:aContext selector) ifTrue:[
        "/ don't check those - they should do so.
        ^ self
    ].

    mthd := aContext compiledMethod.

    "a quick rejector, to avoid many parses (because parsing is very slow)"
    (mthd sendsAnySelector:checkedSelectors) ifFalse:[^ self].

    ^ super checkMethod: aContext
!

initialize
    super initialize.
    self matcher 
            matches: '`{:node | node isMessage 
                                and: [ ( #(add: addFirst: addLast: addAll: addAllLast:) includes:node selector) 
                                and: [ node isDirectlyUsed ]]
                       }'
            do: [ :node :answer | node ]
! !

!RBUsesAddRule class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

