"{ Encoding: utf8 }"

"
 Copyright (C) 2019 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBParseTreeLintRule subclass:#RBUsesValueOfDoRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Parse-tree Rules'
!

!RBUsesValueOfDoRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) 2019 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBUsesValueOfDoRule methodsFor:'accessing'!

group
    ^ 'Possible Bugs'
!

name
    ^ 'Uses the result of a "do:"-like message'

    "Modified: / 01-03-2012 / 11:27:22 / cg"
!

rationale
    ^ 'Check for possible uses of the result returned by "<code>do:</code>" and similar enumeration messages. 
By convention, these messages return the receiver (usually a collection),
but some seem to return the (last) value from an enumeration,
or - if they forward the do: to an inner collection, they mey even return that one.
<br>
Thus it is not really consistent, what is returned, and your code should not depend on the returned value.'

    "Modified: / 01-03-2012 / 12:17:42 / cg"
!

severity
    "redefine to return one of:
        SeverityInformation
        SeverityError
        SeverityWarning (no need to redefine - except for documentation)
    "

    ^ SeverityInformation

    "Created: / 14-03-2011 / 14:11:02 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!RBUsesValueOfDoRule methodsFor:'initialization'!

checkMethod: aContext
    |mthd checkedSelectors|

    checkedSelectors := #(do: reverseDo: doWithIndex: keysAndValuesDo: with:do:).
    (checkedSelectors includes:aContext selector) ifTrue:[
        "/ don't check those - they should do so.
        ^ self
    ].

    mthd := aContext compiledMethod.

    "a quick rejector, to avoid many parses (because parsing is very slow)"
    (mthd sendsAnySelector:checkedSelectors) ifFalse:[^ self].

    ^ super checkMethod: aContext
!

initialize
    super initialize.
    self matcher 
            matches: '`{:node | 
                            node isMessage 
                                and: [ ( #(do: reverseDo: doWithIndex: keysAndValuesDo: with:do:) includes:node selector)
                                and: [node isDirectlyUsed]]
                       }'
            do: [ :node :answer | node ]
! !

!RBUsesValueOfDoRule class methodsFor:'documentation'!

version_CVS
    ^ '$Header$'
! !

