"{ Encoding: utf8 }"

"
 Copyright (C) 2020 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

RBTransformationRule subclass:#RBWithoutPrefixWithoutSuffixRule
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint-Transformation Rules'
!

!RBWithoutPrefixWithoutSuffixRule class methodsFor:'documentation'!

copyright
"
 Copyright (C) 2020 Claus Gittinger

 Permission is hereby granted, free of charge, to any 
 person obtaining a copy of this software and associated 
 documentation files (the 'Software'), to deal in the 
 Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, 
 sublicense, and/or sell copies of the Software, and to 
 permit persons to whom the Software is furnished to do so, 
 subject to the following conditions:

 The above copyright notice and this permission notice shall 
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, 
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBWithoutPrefixWithoutSuffixRule methodsFor:'accessing'!

group
	^ 'Transformations'
!

name
    ^ '"(x startsWith:y)...(x copyFrom:y size+1) -> "x withoutPrefix:y" (and similar constructs)'

    "Modified: / 12-06-2020 / 15:08:58 / cg"
!

rationale
    ^ 'Checks for prefix or suffix compare, with followup code to remove those,
can be written using <code>withoutPrefix:</code> and <code>withoutSuffix:</code> messages.
<br>I.e.:
<code><pre>
    (<I>coll</I> startsWith:<I>prefix</I>) ifTrue:[<I>coll</I> copyFrom:<I>prefix</I> size+1] ifFalse:[<I>coll</I>]</pre></code> 
can be simplified to:  
<code><pre>
    <I>coll</I> withoutPrefix:<I>prefix</I>
</pre></code>
<P>Similar rewrites are possible for suffix checks.
<P>
The alternatives are easier to read, and there may be tuned versions in the collection protocol.'

    "Modified: / 12-06-2020 / 15:11:42 / cg"
!

severity
        ^ SeverityInformation
! !

!RBWithoutPrefixWithoutSuffixRule methodsFor:'initialization'!

initialize
        |checkLengthLiteralIsStrLiteralSizePlus1|
        
        checkLengthLiteralIsStrLiteralSizePlus1 :=
            [:matchedNode |
                |strLitNode strLit indexLitNode indexLit expected|

                strLitNode := rewriteRule context atPatternVariable:'`#prefConst' ifAbsent:[nil].
                indexLitNode := rewriteRule context atPatternVariable:'`#startIndex' ifAbsent:[nil].
                (strLitNode notNil and:[indexLitNode notNil]) ifTrue:[
                    strLit := strLitNode value.
                    indexLit := indexLitNode value.

                    strLit isString ifTrue:[
                        expected := strLit size + 1
                    ] ifFalse:[
                        strLit isCharacter ifTrue:[
                            expected := 2
                        ]
                    ].
                    expected notNil
                      and:[ indexLit isInteger
                      and:[ indexLit == expected ]]
                ] ifFalse:[
                    false
                ]
            ].
                
        super initialize.
        self rewriteRule
                replace: '(`coll startsWith:`@pref) ifTrue:[ `coll withoutPrefix: `@pref ] ifFalse:[ `coll ]'
                with: '(`coll withoutPrefix:`@pref)';

                replace: '(`coll startsWith:`@pref) ifTrue:[ `coll copyButFirst: `@pref size ] ifFalse:[ `coll ]'
                with: '(`coll withoutPrefix:`@pref)';

                replace: '(`coll startsWith:`@pref) ifTrue:[ `coll copyFrom: `@pref size + 1 ] ifFalse:[ `coll ]'
                with: '(`coll withoutPrefix:`@pref)';

                replace: '(`coll startsWith:`@pref) ifTrue:[ `coll copyFrom: `@pref size + 1 to: `coll size] ifFalse:[ `coll ]'
                with: '(`coll withoutPrefix:`@pref)';

                replace: '(`coll startsWith:`@pref) ifTrue:[ `coll := `coll withoutPrefix: `@pref ]'
                with: '(`coll := `coll withoutPrefix:`@pref)';

                replace: '(`coll startsWith:`@pref) ifTrue:[ `coll := `coll copyButFirst: `@pref ]'
                with: '(`coll := `coll withoutPrefix:`@pref)';

                replace: '(`coll startsWith:`@pref) ifTrue:[ `coll := `coll copyFrom: `@pref size + 1 ]'
                with: '(`coll := `coll withoutPrefix:`@pref)';

                replace: '(`coll startsWith:`@pref) ifTrue:[ `coll := `coll copyFrom: `@pref size + 1 to: `coll size]'
                with: '(`coll := `coll withoutPrefix:`@pref)';

                replace: '(`coll startsWith:`#prefConst) ifTrue:[ `coll copyFrom: `#startIndex ] ifFalse:[ `coll ]'
                with: '(`coll withoutPrefix:`#prefConst)'
                when:checkLengthLiteralIsStrLiteralSizePlus1;

                replace: '(`coll startsWith:`#prefConst) ifTrue:[ `coll copyFrom: `#startIndex to: `coll size ] ifFalse:[ `coll ]'
                with: '(`coll withoutPrefix:`#prefConst)'
                when:checkLengthLiteralIsStrLiteralSizePlus1;

                replace: '(`coll startsWith:`#prefConst) ifTrue:[ `coll := `coll copyFrom: `#startIndex ]'
                with: '(`coll := `coll withoutPrefix:`#prefConst)'
                when:checkLengthLiteralIsStrLiteralSizePlus1;

                replace: '(`coll startsWith:`#prefConst) ifTrue:[ `coll := `coll copyFrom: `#startIndex to: `coll size ]'
                with: '(`coll := `coll withoutPrefix:`#prefConst)'
                when:checkLengthLiteralIsStrLiteralSizePlus1;


                replace: '(`coll endsWith:`@suff) ifTrue:[ `coll withoutSuffix: `@suff ] ifFalse:[ `coll ]'
                with: '(`coll withoutSuffix:`@suff)';

                replace: '(`coll endsWith:`@suff) ifTrue:[ `coll copyButLast: `@suff size ] ifFalse:[ `coll ]'
                with: '(`coll withoutSuffix:`@suff)';

                replace: '(`coll endsWith:`@suff) ifTrue:[ `coll copyTo: `coll size - `@suff size] ifFalse:[ `coll ]'
                with: '(`coll withoutSuffix:`@suff)';

                replace: '(`coll endsWith:`@suff) ifTrue:[ `coll copyFrom: 1 to: `coll size - `@suff size] ifFalse:[ `coll ]'
                with: '(`coll withoutSuffix:`@suff)';

                replace: '(`coll endsWith:`@suff) ifTrue:[ `coll := `coll withoutSuffix: `@suff ]'
                with: '(`coll := `coll withoutSuffix:`@suff)';

                replace: '(`coll endsWith:`@suff) ifTrue:[ `coll := `coll copyButLast: `@suff size ]'
                with: '(`coll := `coll withoutSuffix:`@suff)';

                replace: '(`coll endsWith:`@suff) ifTrue:[ `coll := `coll copyTo: `coll size - `@suff size]'
                with: '(`coll := `coll withoutSuffix:`@suff)';

                replace: '(`coll endsWith:`@suff) ifTrue:[ `coll := `coll copyFrom: 1 to: `coll size - `@suff size]'
                with: '(`coll := `coll withoutSuffix:`@suff)';


                yourself.

    "Modified: / 21-07-2020 / 14:13:01 / cg"
! !

!RBWithoutPrefixWithoutSuffixRule methodsFor:'running'!

checkMethod: aContext
    |selector|

    selector := aContext selector.

    "/ skip methods which implement one of the transform targets.
    ( #( withoutPrefix: withoutSuffix: ) includes: selector) 
    ifTrue:[^ self].

    super checkMethod: aContext

    "Modified: / 12-06-2020 / 15:12:09 / cg"
! !

!RBWithoutPrefixWithoutSuffixRule class methodsFor:'documentation'!

version_CVS
    ^ '$Header$'
! !

