"
COPYRIGHT (c) 2022 LabWare


 Permission is hereby granted, free of charge, to any
 person obtaining a copy of this software and associated
 documentation files (the 'Software'), to deal in the
 Software without restriction, including without limitation
 the rights to use, copy, modify, merge, publish, distribute,
 sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so,
 subject to the following conditions:

 The above copyright notice and this permission notice shall
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/lint' }"

"{ NameSpace: Smalltalk }"

Object subclass:#SmalllintChecker
	instanceVariableNames:'rule environment context methodBlock'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Lint'
!

!SmalllintChecker class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2022 LabWare


 Permission is hereby granted, free of charge, to any
 person obtaining a copy of this software and associated
 documentation files (the 'Software'), to deal in the
 Software without restriction, including without limitation
 the rights to use, copy, modify, merge, publish, distribute,
 sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so,
 subject to the following conditions:

 The above copyright notice and this permission notice shall
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!SmalllintChecker class methodsFor:'instance creation'!

new
    ^ self basicNew initialize.
!

newWithContext
	^(self new)
		context: SmalllintContext new;
		yourself
!

runRule: aLintRule
	(self new)
		rule: aLintRule;
		run.
	^aLintRule
!

runRule: aLintRule onEnvironment: aBrowserEnvironment 
        (self new)
                rule: aLintRule;
                environment: aBrowserEnvironment;
                run.
        ^aLintRule
!

runRule: aLintRule onEnvironment: aBrowserEnvironment progressFeedBackInto:aBlockOrNil
        (self new)
                rule: aLintRule;
                environment: aBrowserEnvironment;
                runWithProgressFeedBackInto:aBlockOrNil.
        ^aLintRule
! !

!SmalllintChecker class methodsFor:'others'!

version_CVS
    ^'$Id$'
! !

!SmalllintChecker methodsFor:'accessing'!

context: aSmalllintContext 
	context := aSmalllintContext
!

environment: aBrowserEnvironment 
	environment := aBrowserEnvironment
!

methodBlock: aBlock 
	methodBlock := aBlock
!

rule: aLintRule 
	rule := aLintRule
! !

!SmalllintChecker methodsFor:'actions'!

run
    self runWithProgressFeedBackInto:nil
!

runWithProgressFeedBackInto:feedBackBlockOrNil
    rule resetResult.
    rule recursiveInitialize.

    environment classesDo: [:aClass |
        Error handle:[:ex |
            (Dialog confirm:('Error in smallLint check (%1). Skip check or Debug?' bindWith:rule class name)
                   yesLabel:'Debug' noLabel:'Skip'
            ) ifTrue:[ex reject].
            ex return.    
        ] do:[    
            feedBackBlockOrNil notNil ifTrue:[
                feedBackBlockOrNil value:aClass.
            ].
            self checkClass: aClass.
            self checkMethodsForClass: aClass
        ]
    ]
! !

!SmalllintChecker methodsFor:'initialize-release'!

initialize
	methodBlock := [].
	environment := SelectorEnvironment new.
	context := SmalllintContext newNoCache
!

release
	context release.
	super release
! !

!SmalllintChecker methodsFor:'private'!

checkClass: aClass
    context environment: environment.
    [
        context selectedClass: aClass.

        "/ skip when doing method checks
        "/ I am not sure if this is right, but some class checks take literally hours,
        "/ even though only a single method is checked.
        environment isSelector ifTrue:[^ self].

        (environment definesClass: aClass) ifTrue: [
            rule checkClass: context
        ]
    ] ensure:[
        context selectedClass: nil
    ]

    "Modified (format): / 19-11-2016 / 19:20:11 / cg"
    "Modified: / 16-08-2022 / 15:07:47 / Jan Vrany <jan.vrany@labware.com>"
!

checkMethodsForClass: aClass
    context environment: environment.
    environment 
        selectorsForClass: aClass
        do:[:eachSelector | 
            |replacements|

            (rule shouldIgnoreClass:aClass selector:eachSelector) ifTrue:[
                Transcript showCR:'skipping rule ',rule class name,' in ',aClass name,'>>',eachSelector.
            ] ifFalse:[    
                [
                    context selectedClass: aClass.
                    context selector: eachSelector.
                    replacements := rule checkMethodReturningReplacements: context.
                    replacements notEmptyOrNil ifTrue:[
                        "/ self breakPoint:#cg.
                        rule forgetReplacements.
                    ].
                    methodBlock value.
                ] ensure: [ 
                    context selectedClass: nil.
                    context selector: nil.
                ].
            ]
        ]

    "Modified: / 24-08-2010 / 21:32:39 / Jan Vrany"
    "Modified (format): / 19-11-2016 / 19:19:37 / cg"
    "Modified: / 16-08-2022 / 15:07:40 / Jan Vrany <jan.vrany@labware.com>"
! !

!SmalllintChecker class methodsFor:'documentation'!

version
    ^'$Header$'
! !

