"{ Encoding: utf8 }"

"{ Package: 'stx:goodies/refactoryBrowser/parser' }"

"{ NameSpace: Smalltalk }"

RBValueToken subclass:#RBLiteralToken
	instanceVariableNames:'stopPosition originalString'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Scanner'
!

!RBLiteralToken class methodsFor:'documentation'!

documentation
"
    documentation to be added.

    [author:]
        Claus Gittinger

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!RBLiteralToken class methodsFor:'instance creation'!

value: anObject 
        | literal |
        literal := anObject isArray 
                                ifTrue: [anObject collect: [:each | self value: each]]
                                ifFalse: [anObject].
        ^self 
                value: literal
                start: nil
                stop: nil
!

value: aString start: startInteger stop: stopInteger 
        ^self new
                value: aString
                start: startInteger
                stop: stopInteger
!

value: aString start: anInteger stop: stopInteger lineNumber:lineNumber
        ^self new
                value: aString
                start: anInteger
                stop: stopInteger
                lineNumber:lineNumber
! !

!RBLiteralToken methodsFor:'accessing'!

originalString
    "to remember the original string from scanning
     (in case of a radix-number or a c-string)"

    ^ originalString

    "Modified (comment): / 15-02-2019 / 16:39:58 / Claus Gittinger"
!

originalString:aString
    "to remember the original string from scanning
     (in case of a radix-number or a c-string)"

    originalString := aString.

    "Modified (comment): / 15-02-2019 / 16:39:53 / Claus Gittinger"
!

radix
    ^ nil
!

realValue
        ^value isArray
                ifTrue: [value collect: [:each | each realValue]]
                ifFalse: [value]
!

stop: stopInteger 
        self assert:stopInteger notNil.
        stopPosition := stopInteger
! !

!RBLiteralToken methodsFor:'initialization & release'!

start: startInteger stop: stopInteger 
        "/ self assert:startInteger notNil.
        "/ self assert:stopInteger notNil.

        sourcePointer := startInteger.
        stopPosition := stopInteger
!

value: aString start: startInteger stop: stopInteger 
        "/ self assert:startInteger notNil.
        "/ self assert:stopInteger notNil.

        originalString := nil. "/ flush.
        
        value := aString.
        sourcePointer := startInteger.
        stopPosition := stopInteger.

    "Modified (format): / 15-02-2019 / 16:56:38 / Claus Gittinger"
!

value: aString start: startInteger stop: stopInteger lineNumber:lineNumberInteger
        "/ self assert:startInteger notNil.
        "/ self assert:stopInteger notNil.

        originalString := nil. "/ flush.

        value := aString.
        sourcePointer := startInteger.
        stopPosition := stopInteger.
        lineNumber := lineNumberInteger.
        "/ self assert:(lineNumber notNil).

    "Modified: / 15-02-2019 / 16:56:34 / Claus Gittinger"
! !

!RBLiteralToken methodsFor:'printing'!

printValueOn: aStream
        (originalString ? value) storeOn: "printOn:" aStream.

    "Created: / 15-06-2020 / 17:12:20 / cg"
! !

!RBLiteralToken methodsFor:'private'!

length
        ^stopPosition - (self start ? 1) + 1
! !

!RBLiteralToken methodsFor:'testing'!

isLiteral
	^true
!

isLiteralArray
        ^ value isArray
!

isLiteralByteArray
        ^ value class == ByteArray
!

isLiteralCString
    ^ self isSTXSpecialLiteralString:$c
!

isLiteralCharacter
        ^ value class == Character
!

isLiteralEString
    ^ self isSTXSpecialLiteralString:$e
!

isLiteralFloat
        ^ value isFloat
!

isLiteralIString
    ^ self isSTXSpecialLiteralString:$i
!

isLiteralInteger
        ^ value isInteger
!

isLiteralNil
        ^ value == nil

    "Created: / 06-02-2017 / 14:57:56 / cg"
!

isLiteralRString
    ^ self isSTXSpecialLiteralString:$r

    "Created: / 17-02-2019 / 12:48:41 / Claus Gittinger"
!

isLiteralString
        ^ value isKindOf: String

    "Created: / 28-12-2011 / 16:07:12 / cg"
!

isLiteralSymbol
        ^ value isSymbol

    "Created: / 28-12-2011 / 16:07:12 / cg"
!

isPatternLiteral
        ^false
!

isSTXSpecialLiteralString:typeChar
    ^ originalString notNil
      and:[ self isLiteralString
      and:[ (originalString size > 2)
      and:[ (originalString at:1) == typeChar
      and:[ (originalString at:2) == $'
    ]]]]

    "Created: / 17-02-2019 / 12:48:41 / Claus Gittinger"
! !

!RBLiteralToken class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

