"{ Encoding: utf8 }"

"{ Package: 'stx:goodies/refactoryBrowser/parser' }"

"{ NameSpace: Smalltalk }"

RBProgramNode subclass:#RBMethodNode
	instanceVariableNames:'selector selectorParts body source arguments tags replacements
		nodeReplacements pragmas'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-AST-Nodes'
!

RBMethodNode comment:''
!


!RBMethodNode class methodsFor:'instance creation'!

selector: aSymbol arguments: variableNodes body: aSequenceNode 
	^(self new)
		arguments: variableNodes;
		selector: aSymbol;
		body: aSequenceNode;
		yourself
!

selector: aSymbol body: aSequenceNode 
	^self 
		selector: aSymbol
		arguments: #()
		body: aSequenceNode
!

selectorParts: tokenCollection arguments: variableNodes 
        ^((tokenCollection contains: [:each | each isPatternVariable]) 
                ifTrue: [RBPatternMethodNode] 
                ifFalse: [RBMethodNode]
        ) new 
            selectorParts: tokenCollection arguments: variableNodes

    "Modified: / 24-05-2018 / 20:52:11 / Claus Gittinger"
! !

!RBMethodNode methodsFor:'accessing'!

addNode: aNode 
	^body addNode: aNode
!

addReturn
	body addReturn
!

addSelfReturn
	^body addSelfReturn
!

allArgumentVariablesInto:aCollection
        aCollection addAll:(self argumentNames).
        super allArgumentVariablesInto:aCollection

    "Created: / 23-11-2017 / 22:05:36 / cg"
    "Modified: / 20-03-2018 / 14:27:42 / mawalch"
!

allDefinedVariables
	^(self argumentNames asOrderedCollection) addAll: super allDefinedVariables;
		yourself
!

allVariablesOnScope
    ^ self argumentNames "allArgumentVariables" , (super allVariablesOnScope)

    "Created: / 15-10-2006 / 14:05:08 / cg"
!

argumentNames
        ^ arguments collect: [:each | each name]
!

arguments
	^arguments
!

arguments: variableNodes 
	arguments := variableNodes.
	arguments do: [:each | each parent: self]
!

blockScope
        ^nil

    "Created: / 01-05-2016 / 15:53:43 / cg"
!

body
	^body
!

body: stmtsNode 
	body := stmtsNode.
	body parent: self
!

children
    body isNil ifTrue:[ ^ arguments ].
    ^ arguments copyWith: body
!

methodScope
        ^self

    "Created: / 01-05-2016 / 15:54:18 / cg"
!

newSource
    "return a (horribly formatted) new source"

    replacements isNil ifTrue:[ ^ self formattedCode].
    ^ [self reformatSource] on: Error do:[:ex | ex return:self formattedCode].
!

pragmas
    ^ pragmas ? #()

    "Created: / 07-10-2014 / 23:26:32 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

pragmas: aCollection
    pragmas := aCollection.
    pragmas do: [ :each | each parent: self ]

    "Created: / 07-10-2014 / 23:26:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

primitiveSources
    "returns a oollection of the resources/tags strings"
    
    ^self tags collect: [:each | self source copyFrom: each first to: each last]

    "Modified (comment): / 06-02-2017 / 16:01:11 / cg"
!

renameSelector: newSelector andArguments: varNodeCollection 
        | oldSelectorParts oldArguments |
        oldSelectorParts := selectorParts.
        oldArguments := arguments.
        self
                arguments: varNodeCollection;
                selector: newSelector.
        self changeSourceSelectors: oldSelectorParts arguments: oldArguments
!

selector
	^selector isNil
		ifTrue: [selector := self buildSelector]
		ifFalse: [selector]
!

selector: aSelector 
        | keywords numArgs |

        keywords := aSelector keywords.
        numArgs := aSelector argumentCount.
        numArgs == arguments size ifFalse:[
            self error: 'Attempting to assign selector with wrong number of arguments.'
        ].

        numArgs == 0 ifTrue: [ 
            selectorParts := { RBIdentifierToken value: keywords first start: nil } 
        ] ifFalse:[
            keywords first last == $: ifTrue: [ 
                selectorParts := keywords collect: [:each | RBKeywordToken value: each start: nil]
            ] ifFalse: [ 
                selectorParts := { RBBinarySelectorToken value: aSelector start: nil} 
            ]
        ].
        selector := aSelector
!

source
	^source
!

source: anObject
	source := anObject
!

start
	^1
!

stop
	^source size
!

tags
	^tags isNil ifTrue: [#()] ifFalse: [tags]
!

tags: aCollectionOfIntervals 
	tags := aCollectionOfIntervals
!

temporaries
    body isNil ifTrue:[^ #()].
    ^ body temporaries
! !

!RBMethodNode methodsFor:'comparing'!

= anObject 
        self == anObject ifTrue: [^true].
        self class = anObject class ifFalse: [^false].
        (self selector = anObject selector) ifFalse: [^false].
        (body = anObject body) ifFalse: [^false].
        1 to: arguments size
                do: [:i | (arguments at: i) = (anObject arguments at: i) ifFalse: [^false]].
        ^true
!

equalTo: anObject withMapping: aDictionary 
    "return true, if I am equal to another node, with a given variable name mapping.
     Side effect: update the mapping dictionary as required to reflect a matching mapping (if possible)"

    self class = anObject class ifFalse: [^false].
    self selector = anObject selector ifFalse: [^false].

    "/ original checked the body first, then the args:
    false ifTrue:[
        (body equalTo: anObject body withMapping: aDictionary) ifFalse: [^ false].
        arguments doWithIndex:[:argi :i | 
            (argi equalTo: (anObject arguments at: i) withMapping: aDictionary) ifFalse: [
                ^ false
            ].
            aDictionary removeKey: argi name
        ].
    ].
    
    true ifTrue:[
        "/ that was wrong, because if an argument is shadowed by a local,
        "/ the body wouldn't know, that it does so, and treat the shadowd local as the being same as the arg
        "/ therefore, first check the arguments (adding mappings),
        "/ then the body.
        arguments doWithIndex:[:argi :i |
            |otherArgi|

            otherArgi := (anObject arguments at: i).
            self assert:(aDictionary includesKey:argi name) not.
            self assert:(aDictionary includesValue:otherArgi name) not.

            (argi equalTo: (anObject arguments at: i) withMapping: aDictionary) ifFalse: [
                ^ false
            ].
        ].
        "/ now dictionary contains mappings for the args
        (body equalTo: anObject body withMapping: aDictionary) ifFalse: [^ false].
        arguments do:[:argi |
            aDictionary removeKey: argi name ifAbsent:[]
        ].
    ].
    
    ^ self primitiveSources = anObject primitiveSources

    "Modified: / 14-02-2017 / 13:11:57 / cg"
!

equalTo: anObject withSelectorMapping: aDictionary 
    "return true, if I am equal to another node, with a given selector name mapping.
     Side effect: update the mapping dictionary as required to reflect a matching mapping (if possible)"

    self class = anObject class ifFalse: [^false].
    (aDictionary at:self selector ifAbsentPut:[anObject selector]) = anObject selector ifFalse:[
        ^ false
    ].
    "/ CG: what about the arguments here?
    #FIXME.
    
    (body equalTo: anObject body withSelectorMapping: aDictionary) ifFalse: [^false].
    ^self primitiveSources = anObject primitiveSources

    "Modified (format): / 06-02-2017 / 16:18:01 / cg"
!

hash
        ^(self selector hash bitXor: body hash)
                bitXor: arguments hash
!

semanticallyEqualTo: anObject withMapping: aDictionary 
    "return true, if I am semantically equal to another node, with a given variable name mapping.
     Side effect: update the mapping dictionary as required to reflect a matching mapping (if possible)"

    ^ self semanticallyEqualTo: anObject withMapping: aDictionary ignoringSelector:false

    "Modified: / 05-02-2017 / 00:55:40 / cg"
!

semanticallyEqualTo: anObject withMapping: aDictionary ignoringSelector:ignoringSelector
    "return true, if I am semantically equal to another node, with a given variable name mapping.
     Side effect: update the mapping dictionary as required to reflect a matching mapping (if possible)"

    ^ self
        semanticallyEqualTo: anObject withMapping: aDictionary 
        ignoringSelector:ignoringSelector
        ignoringAnnotations:false

    "Created: / 05-02-2017 / 00:55:18 / cg"
    "Modified: / 06-02-2017 / 10:22:23 / cg"
!

semanticallyEqualTo: anObject withMapping: aDictionary ignoringSelector:ignoringSelector ignoringAnnotations:ignoringAnnotations
    "return true, if I am semantically equal to another node, with a given variable name mapping.
     Side effect: update the mapping dictionary as required to reflect a matching mapping (if possible)"

    |mySelector otherSelector|

    self class = anObject class ifFalse: [^false].

    mySelector := self selector.
    otherSelector := anObject selector.

    ignoringSelector ifTrue:[
        (mySelector argumentCount == otherSelector argumentCount) ifFalse: [^false].  
    ] ifFalse:[    
        (mySelector = otherSelector) ifFalse: [^false].  
    ].
    arguments with:anObject arguments do:[:argHere :argThere | 
        aDictionary at:argHere name put:argThere name
    ].
    
    (body semanticallyEqualTo: anObject body withMapping: aDictionary) ifFalse: [^false].

    arguments with:anObject arguments do:[:argHere :argThere | 
        (argHere 
                semanticallyEqualTo:argThere
                withMapping: aDictionary
        ) ifFalse: [
            ^false
        ].
    ].
    "/ order?
    arguments doWithIndex:[:argName1 :argIndex|
        |argName2|

        argName2 := aDictionary at:argName1 name ifAbsent:argName1.
        (anObject arguments findFirst:[:arg | arg name = argName2]) = argIndex ifFalse:[^ false].
    ].
    
    arguments do:[:argHere |
        |name|
        
        name := argHere name.
        ((aDictionary at:name ifAbsent:name) = name) ifTrue:[
            aDictionary removeKey:name
        ].    
    ].
    ignoringAnnotations ifFalse:[
        "/ actually comparing resources and annotations here.
        ^ self primitiveSources = anObject primitiveSources
    ].
    ^ true.

    "Created: / 06-02-2017 / 10:21:51 / cg"
    "Modified: / 06-02-2017 / 16:03:57 / cg"
! !

!RBMethodNode methodsFor:'copying-private'!

postCopy
        super postCopy.
        self body: body copy.
        self arguments: (arguments collect: [:each | each copy])

    "Modified: / 18-06-2015 / 20:36:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !


!RBMethodNode methodsFor:'enumeration'!

variableNodesDo:aBlock
    "evaluate a Block for all variable nodes"

    arguments do:[:each | 
        each isPatternNode ifFalse:[ aBlock value: each]
    ].
    body notNil ifTrue:[ body variableNodesDo:aBlock].

    "Modified: / 16-11-2016 / 18:55:31 / cg"
! !

!RBMethodNode methodsFor:'initialization & release'!

selectorParts: tokenCollection arguments: variableNodes 
	selectorParts := tokenCollection.
	self arguments: variableNodes
! !

!RBMethodNode methodsFor:'matching'!

copyInContext: aDictionary 
	^(self class new)
		selectorParts: (selectorParts collect: [:each | each removePositions]);
		arguments: (arguments collect: [:each | each copyInContext: aDictionary]);
		body: (body copyInContext: aDictionary);
		source: (aDictionary at: '-source-');
		yourself
!

match: aNode inContext: aDictionary 
        self species == aNode species ifFalse: [^false].
        aDictionary at: '-source-' put: aNode source.
        self selector = aNode selector ifFalse: [^false].
        ^(self matchList: arguments
                against: aNode arguments
                inContext: aDictionary)
                        and: [body match: aNode body inContext: aDictionary]

    "Modified: / 06-02-2017 / 14:35:27 / cg"
! !

!RBMethodNode methodsFor:'printing'!

printOn: aStream
	aStream nextPutAll: self formattedCode
! !

!RBMethodNode methodsFor:'private'!

buildSelector
    "to be checked: this creates symbols, although it probably shouldn't"
    
    | selectorStream |

    selectorStream := WriteStream on: (String new: 50).
    selectorParts do: [:each | selectorStream nextPutAll: each value].
    ^ selectorStream contents asSymbol

    "Modified (comment): / 06-02-2017 / 13:03:00 / cg"
!

changeSourceSelectors: oldSelectorParts arguments: oldArguments
        "If this is the same number of arguments, we try a one to one replacement of selector parts and arguments. 
         If this is not the case try to rewrite the signature as a whole, 
         what unfortunately drops the comments within the signature."

        [ (oldSelectorParts size == selectorParts size and: [ oldArguments size == arguments size ])
                ifTrue: [
                        oldSelectorParts with: selectorParts do: [ :old :new | 
                                self addReplacement: (RBStringReplacement 
                                        replaceFrom: old start
                                        to: old stop
                                        with: new value) ].
                        oldArguments with: arguments do: [ :old :new | 
                                self addReplacement: (RBStringReplacement 
                                        replaceFrom: old start
                                        to: old stop
                                        with: new name) ] ]
                ifFalse: [
                        self addReplacement: (RBStringReplacement
                                replaceFrom: oldSelectorParts first start
                                to: (oldArguments notEmpty
                                        ifTrue: [ oldArguments last stop ]
                                        ifFalse: [ oldSelectorParts last stop ])
                                with: (String streamContents: [ :stream |
                                        selectorParts keysAndValuesDo: [ :index :part |
                                                index == 1 ifFalse: [ stream space ].
                                                stream nextPutAll: part value.
                                                index <= arguments size
                                                        ifTrue: [ stream space; nextPutAll: (arguments at: index) name ] ] ])) ] ]
                on: Error
                do: [ :ex | ex return ]
!

reformatSource
        | stream newSource newTree |
        stream := CharacterWriteStream new:(source size + 100).
        stream
                nextPutAll: (source
                        copyFrom: (replacements
                                inject: 1
                                into: [ :sum :each | 
                                        stream
                                                nextPutAll: (source copyFrom: sum to: each startPosition - 1);
                                                nextPutAll: (each string).
                                        each stopPosition + 1 ])
                        to: source size).
        newSource := stream contents.
        newTree := RBParser 
                parseMethod: newSource 
                onError: [ :msg :pos | ^ self formattedCode ].
        self = newTree
                ifFalse: [ ^ self formattedCode ].
        ^ newSource
!

selectorParts
	^selectorParts
!

selectorParts: tokenCollection 
	selectorParts := tokenCollection
! !

!RBMethodNode methodsFor:'replacing'!

replaceNode: aNode withNode: anotherNode 
	aNode == body ifTrue: [self body: anotherNode].
	self arguments: (arguments 
				collect: [:each | each == aNode ifTrue: [anotherNode] ifFalse: [each]])
! !

!RBMethodNode methodsFor:'testing'!

defines: aName 
        ^ arguments contains: [:each | each name = aName]

    "Modified: / 24-05-2018 / 20:52:51 / Claus Gittinger"
!

isLast: aNode 
	^body isLast: aNode
!

isMethod
	^true
!

isPrimitive
    RBParser isSmalltalkX ifTrue:[
        (body notNil and:[body statements contains:[:node | node isSTXPrimitiveCCode]]) ifTrue:[
            ^ true
        ].
    ].
    ^tags notEmptyOrNil 
    and:[ self primitiveSources contains: [:each | '*primitive*' match: each]]

    "Modified: / 06-02-2017 / 16:00:12 / cg"
    "Modified: / 24-05-2018 / 20:52:41 / Claus Gittinger"
!

lastIsLoop
        ^body lastIsLoop

    "Created: / 16-07-2017 / 13:40:38 / cg"
!

lastIsReturn
	^body lastIsReturn
!

references: aVariableName 
	^body references: aVariableName
!

uses: aNode 
	^body == aNode and: [aNode lastIsReturn]
! !

!RBMethodNode methodsFor:'visitor'!

acceptCommentsAfterVisitor:aVisitor
    aVisitor formatMethodCommentsAfter:(self commentsAfter) for:self
!

acceptCommentsBeforeVisitor:aVisitor
    aVisitor formatMethodCommentsBefore:(self commentsBefore) for:self
!

acceptVisitor: aProgramNodeVisitor 
	^aProgramNodeVisitor acceptMethodNode: self
! !

!RBMethodNode class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

