"{ Encoding: utf8 }"

"{ Package: 'stx:goodies/refactoryBrowser/parser' }"

"{ NameSpace: Smalltalk }"

RBStatementNode subclass:#RBValueNode
	instanceVariableNames:'parentheses'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-AST-Nodes'
!

RBValueNode comment:'BRValueNode is an abstract class that represents a node that returns some value.

Instance Variables:
	parentheses	<SequenceableCollection of: Inteval>	the positions of the parethesis around this node. We need a collection of intervals for stupid code such as "((3 + 4))" that has multiple parethesis around the same expression.

'
!


!RBValueNode methodsFor:'accessing'!

addParenthesis: anInterval 
	parentheses isNil ifTrue: [parentheses := OrderedCollection new: 1].
	parentheses add: anInterval
!

leftMostReceiver
    ^ self
!

parentheses
	^parentheses isNil
		ifTrue: [#()]
		ifFalse: [parentheses]
!

start
        ^parentheses isNil 
                ifTrue: [self startWithoutParentheses]
                ifFalse: [parentheses last first]
!

startWithoutParentheses
	^self subclassResponsibility
!

stop
	^parentheses isNil
		ifTrue: [self stopWithoutParentheses]
		ifFalse: [parentheses last last]
!

stopWithoutParentheses
	^self subclassResponsibility
! !

!RBValueNode methodsFor:'comparing'!

semanticallyEqualToNegated:expr2 withMapping:mappingDict
    "/ return true,
    "/ if the receiver and expr2 compute the negated result of each other;
    "/ eg. '(a > b)' vs. '(a > b) not'

    |savMapping|

    (expr2 isMessage and:[expr2 selector = 'not']) ifTrue:[
        savMapping := mappingDict copy.
        (self semanticallyEqualTo:expr2 receiver withMapping:mappingDict) ifTrue:[^ true].
        "/ undo mappings.
        mappingDict clearContents; declareAllFrom:savMapping.
    ].
    ^ false

    "Created: / 06-02-2017 / 13:33:45 / cg"
! !

!RBValueNode methodsFor:'testing'!

containedBy: anInterval 
    "true if the interval contains me completely"

        ^anInterval first <= self startWithoutParentheses 
                and: [anInterval last >= self stopWithoutParentheses]
!

hasParentheses
        ^ parentheses notEmptyOrNil
!

isValue
	^true
! !

!RBValueNode class methodsFor:'documentation'!

version
    ^ '$Header$'
! !

