"
COPYRIGHT (c) 2022 LabWare


 Permission is hereby granted, free of charge, to any
 person obtaining a copy of this software and associated
 documentation files (the 'Software'), to deal in the
 Software without restriction, including without limitation
 the rights to use, copy, modify, merge, publish, distribute,
 sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so,
 subject to the following conditions:

 The above copyright notice and this permission notice shall
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/parser' }"

"{ NameSpace: Smalltalk }"

RBValueNode subclass:#RBVariableNode
	instanceVariableNames:'token'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-AST-Nodes'
!

RBVariableNode comment:'BRVariableNode is an AST node that represent a variable (global, inst var, temp, etc.).

Instance Variables:
	token	<BRValueToken>	the token that contains our name and position

'
!

!RBVariableNode class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2022 LabWare


 Permission is hereby granted, free of charge, to any
 person obtaining a copy of this software and associated
 documentation files (the 'Software'), to deal in the
 Software without restriction, including without limitation
 the rights to use, copy, modify, merge, publish, distribute,
 sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so,
 subject to the following conditions:

 The above copyright notice and this permission notice shall
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
! !

!RBVariableNode class methodsFor:'instance creation'!

identifierToken: anIdentifierToken 
	^(anIdentifierToken isPatternVariable 
		ifTrue: [RBPatternVariableNode]
		ifFalse: [RBVariableNode]) new 
		identifierToken: anIdentifierToken
!

named: aString 
        ^self identifierToken: (RBIdentifierToken value: aString start: 0)
! !

!RBVariableNode methodsFor:'accessing'!

addReferencedVariablesTo:aCollection
    aCollection add:self
!

definingScope
    "Return the sequenceNode, blockNode or MethodNode (i.e. the scope) 
     that defines this variable or nil, if none does"

    | owner |

    owner := parent.
    [ owner notNil ] whileTrue:[
        (owner defines: self name) ifTrue:[ ^ owner ].
        owner := owner parent.
    ].
    ^nil

    "Created: / 03-02-2019 / 15:26:02 / Claus Gittinger"
!

definingSequence
    "Return the sequenceNode (i.e. the scope) that defines this variable
     or nil, if no sequenceNode does.
     WARNING: this seems to be misused (or misdefined), in that it will only
     look for sequenceNode scopes (i.e. temporary variables),
     NOT for arguments.
     Use definingScope to get to those as well.
     (I am not sure, if this should be fixed here, 
      or not, to prevent breaking existing code.
      Therefore this remains as is)"

    | owner |

    owner := parent.
    [ owner notNil ] whileTrue:[
        owner isSequence ifTrue:[
            (owner defines: self name) ifTrue:[ ^ owner ]
        ].
        owner := owner parent.
    ].
    ^nil

    "Created: / 06-09-2012 / 16:58:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (comment): / 03-02-2019 / 15:27:42 / Claus Gittinger"
!

lineNumber
    lineNumber isNil ifTrue:[
        lineNumber := token lineNumber.
        lineNumber isNil ifTrue:[self halt].
    ].
    ^ lineNumber

    "Created: / 29-11-2017 / 20:08:28 / cg"
!

name
    token isOldStyleAssignment ifTrue: [ ^ '_' ].
    ^token value

    "Modified: / 31-10-2022 / 16:06:58 / Jan Vrany <jan.vrany@labware.com>"
!

name: aString
        ^token value: aString

    "Created: / 19-06-2015 / 07:03:46 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

precedence
	^0
!

startWithoutParentheses
	^token start
!

stopWithoutParentheses
	^token stop
!

token
    ^ token

    "Created: / 07-10-2014 / 23:27:05 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

tokenPosition
        ^token position
! !

!RBVariableNode methodsFor:'comparing'!

= anObject 
	self == anObject ifTrue: [^true].
	self class = anObject class ifFalse: [^false].
	^self name = anObject name
!

equalTo: anObject withMapping: aDictionary 
        "return true, if I am equal to another node, with a given variable name mapping.
         Side effect: update the mapping dictionary as required to reflect a matching mapping (if possible)"

        |otherName|
        self class = anObject class ifFalse: [^false].
        otherName := anObject name.
        ^(aDictionary at: self name ifAbsentPut: [otherName]) = otherName
!

equalTo: anObject withSelectorMapping: aDictionary 
        "return true, if I am equal to another node, with a given selector name mapping.
         Side effect: update the mapping dictionary as required to reflect a matching mapping (if possible)"

        self class = anObject class ifFalse: [^false].
        ^ self name = anObject name.
!

hash
	^self name hash
! !

!RBVariableNode methodsFor:'copying'!

postCopy
    super postCopy.
    token := token copy.

    "Created: / 28-07-2015 / 06:20:13 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!RBVariableNode methodsFor:'enumeration'!

variableNodesDo:aBlock
    "evaluate a Block for all variable nodes."

    aBlock value:self

    "Modified: / 16-11-2016 / 20:01:57 / cg"
! !

!RBVariableNode methodsFor:'initialization & release'!

identifierToken: anIdentifierToken 
	token := anIdentifierToken
! !

!RBVariableNode methodsFor:'libtool3 support'!

highlightInCodeView: aCodeView

    ^aCodeView highlightVariableNode: self

    "Created: / 18-02-2008 / 17:51:44 / janfrog"
!

leftClickMenuInCodeView: aCodeView 
    ^ aCodeView leftClickMenuForVariableNode: self.

    "Created: / 01-09-2009 / 08:41:17 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

rightClickMenuInCodeView: aCodeView 
    ^ aCodeView rightClickMenuForVariableNode: self.

    "Created: / 18-02-2008 / 21:05:04 / janfrog"
! !

!RBVariableNode methodsFor:'matching'!

copyInContext: aDictionary 
	^self class identifierToken: token removePositions
! !

!RBVariableNode methodsFor:'queries'!

isReferringToAnExceptionInContext:aSearchDictionary
    "true iff:
        (node is a global variable or node is a class variable)
        and the variable refers to an Exception class (but not a Notification)"

    |varName varValue classOfMethod|

    varName := self name.
    
    classOfMethod := aSearchDictionary searcher currentSearchContext selectedClass.
    (classOfMethod theNonMetaclass classVariableNames includes:varName) ifTrue:[
        varValue := classOfMethod theNonMetaclass classVarAt:varName
    ] ifFalse:[   
        varValue := Smalltalk classNamed:varName
    ].
    varValue notNil ifTrue:[
        varValue isBehavior ifTrue:[
            ^ (varValue includesBehavior: Exception) 
            and: [ (varValue includesBehavior: Notification) not ]
        ].
    ].
    ^ false.

    "Created: / 28-11-2017 / 13:01:37 / cg"
    "Modified (comment): / 28-11-2017 / 14:14:41 / cg"
! !

!RBVariableNode methodsFor:'testing'!

isClassVariableInContext:aSearchDictionary
    "not true - also returns true for class vars and globals"

    |varName classOfMethod|
    
    varName := self name.

    self isSelf ifTrue:[^ false].
    self isSuper ifTrue:[^ false].
    self isLocal ifTrue:[^ false].

    classOfMethod := aSearchDictionary searcher currentSearchContext selectedClass.
    ^ classOfMethod notNil
    and:[ (classOfMethod theNonMetaclass classVariableNames includes:varName) ]

    "Created: / 28-11-2017 / 14:15:23 / cg"
!

isGlobal
    "Return true if the variable node is for
     a non-local (not really global), false otherwise.
     Actually wrong: also returns true for inst- and class variables
     (i.e. anything that is not a block- or method local)"

    ^self definingScope isNil

    "Modified: / 03-02-2019 / 15:28:03 / Claus Gittinger"
    "Modified (comment): / 03-02-2019 / 16:28:44 / Claus Gittinger"
!

isImmediate
	^true
!

isInstanceVariable
    "not true - also returns true for class vars and globals"
    
    self isSelf ifTrue:[^ false].
    self isSuper ifTrue:[^ false].
    self isLocal ifTrue:[^ false].
    ^ true
!

isLocal
    "Return true if the variable node is for
     a local variable, false otherwise.
     Returns true for anything that is a block- or method local"

    self isSelf ifTrue:[^ false].
    self isSuper ifTrue:[^ false].
    ^self definingSequence notNil
!

isRead
    "Return true, if the variable is read"

    | root t |

    parent isNil ifTrue:[ ^ false ].
    root := self definingSequence.
    root isNil ifTrue:[
        "OK, not a temporary, search from top-most node"
        [ (t := root parent) notNil ] whileTrue:[ root := t ].
    ].

    ^root reads: self name

    "Created: / 06-09-2012 / 17:13:34 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isSelf
    "true if I am the self-variable node"

    ^ self name = 'self'

    "Created: / 17-07-2011 / 10:28:56 / cg"
!

isSuper
    ^ self name = 'super'

    "Created: / 17-07-2011 / 10:29:29 / cg"
!

isTemporary
    "Return true if the variable node is for
     a temporary variable, false otherwise"

    ^self definingSequence notNil

    "Created: / 06-09-2012 / 16:46:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isVariable
	^true
!

isVariableNotInLeftHandSideOfAssignment
    "Returns true iff receiver is variable node but not
     in left-hand side of assignment"    
    ^self parent isAssignment not 
        or:[self parent variable ~~ self]

    "Created: / 28-03-2012 / 17:15:46 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isWrite
    "Same as isWritten - bad naming here"
    ^self isWritten

    "Modified: / 06-09-2012 / 17:09:37 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isWritten
    "Return true, if the variable is written"

    | root t|

    parent isNil ifTrue:[ ^ false ].
    root := self definingSequence.
    root isNil ifTrue:[
        "OK, not a temporary, search from top-most node"
        [ (t := root parent) notNil ] whileTrue:[ root := t ].
    ].

    ^root assigns: self name

    "Created: / 06-09-2012 / 17:07:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

reads: variableName
    ^self name = variableName

    "Created: / 06-09-2012 / 17:17:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

references: aVariableName 
	^self name = aVariableName
! !

!RBVariableNode methodsFor:'visitor'!

acceptVisitor: aProgramNodeVisitor 
	^aProgramNodeVisitor acceptVariableNode: self
! !

!RBVariableNode class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

