"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011 Claus Gittinger (porting, Smalltalk/X rules & tweaks)

 Permission is hereby granted, free of charge, to any
 person obtaining a copy of this software and associated
 documentation files (the 'Software'), to deal in the
 Software without restriction, including without limitation
 the rights to use, copy, modify, merge, publish, distribute,
 sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so,
 subject to the following conditions:

 The above copyright notice and this permission notice shall
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:goodies/refactoryBrowser/refactoring' }"

"{ NameSpace: Smalltalk }"

LibraryDefinition subclass:#stx_goodies_refactoryBrowser_refactoring
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'* Projects & Packages *'
!

!stx_goodies_refactoryBrowser_refactoring class methodsFor:'documentation'!

copyright
"
 Copyright (C) Original SmallLint Authors
 Copyright (C) The Squeak Team
 Copyright (C) The Pharo Team
 Copyright (C) 2011 Jan Vrany (porting, Smalltalk/X rules & tweaks)
 Copyright (C) 2011 Claus Gittinger (porting, Smalltalk/X rules & tweaks)

 Permission is hereby granted, free of charge, to any
 person obtaining a copy of this software and associated
 documentation files (the 'Software'), to deal in the
 Software without restriction, including without limitation
 the rights to use, copy, modify, merge, publish, distribute,
 sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so,
 subject to the following conditions:

 The above copyright notice and this permission notice shall
 be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"
!

documentation
"
    Package documentation:

    This library contains classes representing individual refactoring operations.
    Refactorings are higher level, and usually more complex changes done to the system,
    often involving multiple smaller changes done to multiple methods or classes.
    Refactorings support undo and redo, and are created and executed via the browser's menu functions.
"
! !

!stx_goodies_refactoryBrowser_refactoring class methodsFor:'description'!

excludedFromPreRequisites
    "list all packages which should be ignored in the automatic
     preRequisites scan. See #preRequisites for more."

    ^ #(
        #'stx:libtool'    "SystemBrowser - referenced by RemoveClassVariableRefactoring>>preconditions "
        #'stx:goodies/refactoryBrowser/browser'    "MethodNameDialog - referenced by Refactoring class>>initializeRefactoringOptions "
    )
!

includedInPreRequisites
    "list all packages which should be also be treated as preRequisite,
     even if not found by the automatic scan. See #preRequisites for more."

    ^ #(
        #'stx:libcompat'    "sends removeClassNamed: from libcompat"
    )
!

mandatoryPreRequisites
    "list packages which are mandatory as a prerequisite.
     This are packages containing superclasses of my classes and classes which
     are extended by myself.
     They are mandatory, because we need these packages as a prerequisite for loading and compiling.
     When loading whole packages,
     mandatoryPreRequisites will be automatically loaded
     BEFORE this packet has been loaded. 
     This method is generated automatically,
     by searching along the inheritance chain of all of my classes.
     Please take a look at the #referencedPreRequisites method as well."

    ^ #(
        #'stx:libbasic'    "Error - superclass of RefactoringError"
    )
!

referencedPreRequisites
    "list packages which are a prerequisite, because they contain
     classes which are referenced by my classes.
     These packages are NOT needed as a prerequisite for compiling or loading,
     however, a class from it may be referenced during execution and having it
     unloaded then may lead to a runtime doesNotUnderstand error, unless the caller
     includes explicit checks for the package being present.
     When loading whole packages,
     referencedPreRequisites will be automatically loaded
     AFTER this packet has been loaded. 
     This method is generated automatically,
     by searching all classes (and their packages) which are referenced by my classes.
     Please also take a look at the #mandatoryPreRequisites method"

    ^ #(
        #'stx:goodies/refactoryBrowser/changes'    "ClassRefactoryChange - referenced by RefactoryBuilder>>addClassVariable:to:"
        #'stx:goodies/refactoryBrowser/helpers'    "BrowserEnvironment - referenced by PushDownInstanceVariableRefactoring>>preconditions"
        #'stx:goodies/refactoryBrowser/lint'    "RBReadBeforeWrittenTester - referenced by ExtractMethodRefactoring>>checkAssignments:"
        #'stx:goodies/refactoryBrowser/parser'    "ParseTreeRewriter - referenced by AbstractClassVariableRefactoring>>abstractClassReferences"
        #'stx:libcomp'    "ParseError - referenced by Refactoring>>execute"
        #'stx:libview'    "Cursor - referenced by Refactoring>>allImplementorsOf:"
    )
! !

!stx_goodies_refactoryBrowser_refactoring class methodsFor:'description - compilation'!

additionalBaseAddressDefinition_bc_dot_mak
    "this is an optional definition, which (if present) may speed up the dll-loading a little
     on win32 systems."

    ^ '
# see stdHeader_bc for LIB_RB_REFACTORING_BASE
LIB_BASE=$(LIB_RB_REFACTORING_BASE)
'
!

stcOptimizationOptions
    "do not inline the not operation here - it is overwritten"

    ^ '+optspace3 -inlinenot'

    "Created: / 23-08-2006 / 01:27:22 / cg"
! !

!stx_goodies_refactoryBrowser_refactoring class methodsFor:'description - contents'!

classNamesAndAttributes
    "lists the classes which are to be included in the project.
     Each entry in the list may be: a single class-name (symbol),
     or an array-literal consisting of class name and attributes.
     Attributes are: #autoload or #<os> where os is one of win32, unix,..."

    ^ #(
        "<className> or (<className> attributes...) in load order"
        RBAbstractCondition
        RBReplaceStringInOriginalSource
        RBReplaceTokenStringInOriginalSource
        Refactoring
        RefactoringError
        RefactoringManager
        RefactoringWarning
        RefactoryBuilder
        #'stx_goodies_refactoryBrowser_refactoring'
        AbstractVariablesRefactoring
        ClassRefactoring
        ExpandReferencedPoolsRefactoring
        MethodRefactoring
        RBAlternativeCondition
        RBCondition
        RBConjunctiveCondition
        RBNegationCondition
        RemoveClassRefactoring
        SplitClassRefactoring
        VariableRefactoring
        AbstractClassVariableRefactoring
        AbstractInstanceVariableRefactoring
        AddClassRefactoring
        AddClassVariableRefactoring
        AddInstanceVariableRefactoring
        AddMethodRefactoring
        ChangeMethodNameRefactoring
        ChildrenToSiblingsRefactoring
        CreateAccessorsForVariableRefactoring
        ExtractMethodRefactoring
        ExtractMethodToComponentRefactoring
        ExtractToTemporaryRefactoring
        InlineAllSelfSendersRefactoring
        InlineMethodRefactoring
        InlineTemporaryRefactoring
        MoveMethodRefactoring
        MoveVariableDefinitionRefactoring
        ProtectInstanceVariableRefactoring
        PullUpClassVariableRefactoring
        PushDownClassVariableRefactoring
        PushDownMethodRefactoring
        PushUpMethodRefactoring
        RemoveClassVariableRefactoring
        RemoveInstanceVariableRefactoring
        RemoveMethodRefactoring
        RenameClassRefactoring
        RenameClassVariableRefactoring
        RenameTemporaryRefactoring
        RenameVariableRefactoring
        RestoringVariableRefactoring
        TemporaryToInstanceVariableRefactoring
        ValueHolderRefactoring
        AddParameterRefactoring
        InlineMethodFromComponentRefactoring
        PullUpInstanceVariableRefactoring
        PushDownInstanceVariableRefactoring
        RemoveParameterRefactoring
        RenameInstanceVariableRefactoring
        RenameMethodRefactoring
        InlineParameterRefactoring
        RenameToInstanceVariableRefactoring
        (CompositeRefactoring autoload)
    )
! !

!stx_goodies_refactoryBrowser_refactoring class methodsFor:'description - project information'!

companyName
    "Return a companyname which will appear in <lib>.rc"

    ^ 'Refactoring Inc. / eXept Software AG'

    "Modified: / 18-11-2016 / 11:53:22 / cg"
!

description
    "Return a description string which will appear in nt.def / bc.def"

    ^ 'Smalltalk/X Refactorings'

    "Modified: / 14-09-2006 / 10:56:40 / cg"
!

legalCopyright
    "Return copyright string which will appear in <lib>.rc"

    ^ 'Copyright Refactoring Inc.\nCopyright Claus Gittinger\nCopyright eXept Software AG 2011'

    "Modified: / 18-11-2016 / 12:15:32 / cg"
!

productName
    "Return a product name which will appear in <lib>.rc"

    ^ 'Smalltalk/X'
! !

!stx_goodies_refactoryBrowser_refactoring class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

