"{ Package: 'stx:goodies/refactoryBrowser/tests' }"

RefactoringTest subclass:#AddParameterTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Testing'
!


!AddParameterTest methodsFor:'failure tests'!

testBadInitializationCode
	self
		shouldFail: (AddParameterRefactoring 
					addParameterToMethod: #name
					in: LintRuleTest
					newSelector: #name:
					initializer: 'foo:');
		shouldFail: (AddParameterRefactoring 
					addParameterToMethod: #name
					in: LintRuleTest
					newSelector: #name:
					initializer: 'foo')
!

testModelBadInitializationCode
	| refactoring |
	model removeClassNamed: #Refactoring.
	refactoring := AddParameterRefactoring 
				model: model
				addParameterToMethod: #name1
				in: LintRuleTest
				newSelector: #name1:
				initializer: 'AddParameterRefactoring new'.
	self shouldFail: refactoring
!

testModelNonExistantName
	| refactoring |
	(model classNamed: #LintRuleTest) removeMethod: #name.
	refactoring := AddParameterRefactoring 
				model: model
				addParameterToMethod: #name
				in: LintRuleTest
				newSelector: #nameNew:
				initializer: 'nil'.
	self shouldFail: refactoring
!

testNonExistantName
	self shouldFail: (AddParameterRefactoring 
				addParameterToMethod: #name1
				in: LintRuleTest
				newSelector: #name1:
				initializer: 'nil')
!

testPrimitiveMethods
	| refactoring |
	(model classNamed: #Object) 
		compile: 'foo <primitive: 100> ^#() primitiveFailed'
		classified: #(#accessing).
	refactoring := AddParameterRefactoring 
				model: model
				addParameterToMethod: #foo
				in: Object
				newSelector: #foo123124321s:
				initializer: '1'.
	self shouldFail: refactoring.
	refactoring := AddParameterRefactoring 
				addParameterToMethod: #at:
				in: Object
				newSelector: #at:foo:
				initializer: '1'.
	self shouldFail: refactoring.
        
! !

!AddParameterTest methodsFor:'set up'!

setUp
	super setUp.
	model := Compiler evaluate: (self readFile: 'AbstractVariableTestData.st')
! !

!AddParameterTest methodsFor:'tests'!

testAddParameterForTwoArgumentMessage
	| refactoring class |
	((Smalltalk respondsTo:#isSmalltalkX) and:[Smalltalk isSmalltalkX]) ifTrue:[
	    RefactoryTestDataApp autoload
	].
	refactoring := AddParameterRefactoring 
				addParameterToMethod: ('called:' , 'on:') asSymbol
				in: RefactoryTestDataApp
				newSelector: #called:bar:on:
				initializer: '#(1.0)'.
	self executeRefactoring: refactoring.
	class := refactoring model classNamed: #RefactoryTestDataApp.
	self 
		assert: (class parseTreeFor: #called:bar:on:) = (RBParser 
						parseMethod: 'called: anObject bar: aBlock on: anObject1
							Transcript
								show: anObject printString;
								cr.
								aBlock value').
	self 
		assert: (class parseTreeFor: #caller) = (RBParser 
						parseMethod: 'caller
							| anObject |
							anObject := 5.
							self 
								called: anObject + 1
								bar: [^anObject] on: #(1.0)').
	self deny: (class directlyDefinesMethod: ('called:' , 'on:') asSymbol)
!

testAddParameterThatReferencesGlobalAndLiteral
	| refactoring class |
	refactoring := AddParameterRefactoring 
				addParameterToMethod: ('test' , 'Foo:') asSymbol
				in: RefactoryTestDataApp
				newSelector: #testFoo:bar:
				initializer: 'OrderedCollection new: 5'.
	self executeRefactoring: refactoring.
	class := refactoring model classNamed: #RefactoryTestDataApp.
	self 
		assert: (class parseTreeFor: #testFoo:bar:) = (RBParser 
						parseMethod: 'testFoo: anObject bar: anObject1
								^self class + anObject').
	self 
		assert: (class parseTreeFor: #callFoo) = (RBParser 
						parseMethod: 'callFoo ^self testFoo: 5 bar: (OrderedCollection new: 5)').
	self deny: (class directlyDefinesMethod: ('test' , 'Foo:') asSymbol)
!

testAddParameterThatReferencesModelGlobal
	| refactoring class |
	refactoring := AddParameterRefactoring 
				model: model
				addParameterToMethod: ('test' , 'Foo:') asSymbol
				in: RefactoryTestDataApp
				newSelector: #testFoo:bar:
				initializer: 'Bar new'.
	self executeRefactoring: refactoring.
	class := refactoring model classNamed: #RefactoryTestDataApp.
	self 
		assert: (class parseTreeFor: #testFoo:bar:) = (RBParser 
						parseMethod: 'testFoo: anObject bar: anObject1
								^self class + anObject').
	self assert: (class parseTreeFor: #callFoo) 
				= (RBParser parseMethod: 'callFoo ^self testFoo: 5 bar: (Bar new)').
	self deny: (class directlyDefinesMethod: ('test' , 'Foo:') asSymbol)
!

testAddParameterThatReferencesSelf
	| refactoring class |
	refactoring _ AddParameterRefactoring
				addParameterToMethod: ('test' , 'Foo:') asSymbol
				in: RefactoryTestDataApp
				newSelector: #testFoo:bar:
				initializer: 'self printString'.
	self executeRefactoring: refactoring.
	class _ refactoring model classNamed: #RefactoryTestDataApp.
	self assert: (class parseTreeFor: #testFoo:bar:)
			= (RBParser parseMethod: 'testFoo: anObject bar: anObject1
								^self class + anObject').
	self assert: (class parseTreeFor: #callFoo)
			= (RBParser parseMethod: 'callFoo ^self testFoo: 5 bar: (self printString)').
	self deny: (class directlyDefinesMethod: ('test' , 'Foo:') asSymbol)
! !

!AddParameterTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/tests/AddParameterTest.st,v 1.4 2003-10-06 14:39:33 cg Exp $'
! !
