"{ Package: 'stx:goodies/refactoryBrowser/tests' }"

"{ NameSpace: Smalltalk }"

TestCase subclass:#CodeDuplicationTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-Testing'
!

!CodeDuplicationTest class methodsFor:'documentation'!

documentation
"
    documentation to be added.

    [author:]
        cg

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!CodeDuplicationTest methodsFor:'tests'!

test01_differentNonLocalName
     |m1 m2|

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo ^ foo' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo ^ bar' selector: #retFoo.
     self deny:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2 ).

    "Created: / 06-02-2017 / 10:11:34 / cg"
!

test02_renamedArg
     |m1 m2|

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:foo ^ foo' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:bar ^ bar' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2 ).

    "Created: / 06-02-2017 / 10:11:44 / cg"
!

test02b_sameMethod
     |m1 m2|

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:foo ^ foo' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:foo ^ foo' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2 ).

    "Created: / 06-02-2017 / 10:11:59 / cg"
!

test03_differentNonLocalName
     |m1 m2|

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:foo ^ inst1' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:bar ^ inst2' selector: #retFoo.
     self deny:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2 ).

    "Created: / 06-02-2017 / 10:12:17 / cg"
!

test04_differentNonLocalName
     |m1 m2|

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:foo ^ inst1' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:foo ^ inst2' selector: #retFoo.
     self deny:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2 ).

    "Created: / 06-02-2017 / 10:12:31 / cg"
!

test05_renamedArgNotUsed
     |m1 m2|

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a _:b ^ a' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a _:c ^ a' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2 ).

    "Created: / 06-02-2017 / 10:13:01 / cg"
!

test06_argumentOrder
     |m1 m2|

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a _:b ^ a' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:b _:a ^ a' selector: #retFoo.
     self deny:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2 ).

    "Created: / 06-02-2017 / 10:13:40 / cg"
!

test07_argumentRenamed
     |m1 m2|

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a _:b ^ a' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:b _:a ^ b' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2 ).

    "Created: / 06-02-2017 / 10:13:32 / cg"
!

test08_differentNameButSameSemantic
     |m1 m2|

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo ^ foo' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retBar ^ foo' selector: #retBar.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2 ).

    "Created: / 06-02-2017 / 10:13:57 / cg"
!

test09_innerArgRenamed
     |m1 m2|

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo ^ [:foo | foo + 1]' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo ^ [:bar | bar + 1]' selector: #retBar.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2 ).
     
    "Created: / 06-02-2017 / 10:14:11 / cg"
!

test10_argumentShadowed
     |node1 node2 m1 m2|

     "/ not equal!!
     node1 := RBParser parseMethod:'retFoo:a ^ a'.
     node2 := RBParser parseMethod:'retFoo:a |a| ^ a'.

     self deny:( node1 equalTo:node2 withMapping:Dictionary new).
     self deny:( node2 equalTo:node1 withMapping:Dictionary new).                  "/ check must be commutative
     self deny:( node1 semanticallyEqualTo:node2 withMapping:Dictionary new).
     self deny:( node1 semanticallyEqualTo:node2 withMapping:Dictionary new).      "/ check must be commutative

     "/ the shadowing of 'a' has a sever effect
     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a |a| ^ a' selector: #retFoo.
     self deny:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2 ).
     self deny:( RBCodeDuplicationRule hasMethod:m2 sameSemanticAs:m1 ).    "/ check must be commutative

     "/ the following shadowing of 'b' has no effect
     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a |b| ^ a' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2 ).
     self assert:( RBCodeDuplicationRule hasMethod:m2 sameSemanticAs:m1 ).  "/ check must be commutative  

    "Created: / 06-02-2017 / 10:14:49 / cg"
    "Modified (comment): / 06-02-2017 / 16:12:02 / cg"
!

test11_annotations
    |t1 t2 m1 m2|

    t1 := RBParser parseMethod:'retFoo:a <resource: #EXPECCO_API>^ a'.
    t2 := RBParser parseMethod:'retFoo:a ^ a'.
    self assert:t1 tags notEmptyOrNil.
    self assert:t2 tags isEmptyOrNil.
    
    m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a <resource: #EXPECCO_API>^ a' selector: #retFoo.
    m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a' selector: #retFoo.
    self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2 ignoreAnnotations:true).

    m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a <resource: #EXPECCO_API>^ a' selector: #retFoo.
    m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a' selector: #retFoo.
    self deny:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2 ignoreAnnotations:false).

    "Created: / 06-02-2017 / 10:19:51 / cg"
    "Modified: / 06-02-2017 / 16:04:58 / cg"
!

test12_ifElseReversed
     |m1 m2|

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a a > 0 ifTrue:[1] ifFalse:[0]' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a a > 0 ifFalse:[0] ifTrue:[1]' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

    "Created: / 06-02-2017 / 10:28:37 / cg"
    "Modified: / 06-02-2017 / 16:09:50 / cg"
!

test13_negationAndIf
     |m1 m2|

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a a > 0 ifTrue:[1] ifFalse:[0]' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a (a > 0) not ifTrue:[0] ifFalse:[1]' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

    "Created: / 06-02-2017 / 10:29:21 / cg"
    "Modified: / 06-02-2017 / 16:09:56 / cg"
!

test14_negationAndIfReversed
     |m1 m2|

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a a > 0 ifTrue:[1] ifFalse:[0]' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a (a > 0) not ifFalse:[1] ifTrue:[0] ' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

    "Created: / 06-02-2017 / 10:30:28 / cg"
    "Modified: / 06-02-2017 / 16:10:01 / cg"
!

test15_isNilNotNil
     |m1 m2|

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a a isNil ifTrue:[1] ifFalse:[0]' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a a isNil ifFalse:[0] ifTrue:[1] ' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a a isNil ifFalse:[0] ifTrue:[1] ' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a a isNil ifTrue:[1] ifFalse:[0] ' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a a isNil ifTrue:[1] ifFalse:[0]' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a a notNil ifTrue:[0] ifFalse:[1] ' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a a isNil ifTrue:[1] ifFalse:[0]' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a a notNil ifFalse:[1] ifTrue:[0] ' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a _:b ^ a isNil = b isNil' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a _:b ^ a notNil not = b isNil' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a _:b ^ a isNil' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a _:b ^ a == nil' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a _:b ^ a notNil' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a _:b ^ a ~~ nil' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a _:b ^ a == nil' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a _:b ^ a isNil' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a _:b ^ a ~~ nil' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a _:b ^ a notNil' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

    "Created: / 06-02-2017 / 11:02:04 / cg"
    "Modified: / 06-02-2017 / 16:10:22 / cg"
!

test16_relops
     |m1 m2|

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a > 0' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a > 0' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ (a <= 0) not' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ (a <= 0) not' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a > 0' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ (a <= 0) not' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ (a <= 0) not' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a > 0' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

    "Created: / 06-02-2017 / 11:57:25 / cg"
    "Modified: / 06-02-2017 / 16:10:27 / cg"
!

test17_relopsAndIf
     |m1 m2|

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a > 0 ifTrue:[1] ifFalse:[2]' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ 0 < a ifTrue:[1] ifFalse:[2]' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a > 0 ifTrue:[1] ifFalse:[2]' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ (a <= 0) not ifTrue:[1] ifFalse:[2]' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a > 0 ifTrue:[1] ifFalse:[2]' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ (a <= 0) ifTrue:[2] ifFalse:[1]' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a isNil ifTrue:[1] ifFalse:[2]' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a notNil ifTrue:[2] ifFalse:[1]' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a isNil ifTrue:[1] ifFalse:[2]' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a notNil ifFalse:[1] ifTrue:[2] ' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a isNil ifTrue:[1] ifFalse:[2]' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a isNil not ifTrue:[2] ifFalse:[1]' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a isNil ifTrue:[1] ifFalse:[2]' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a isNil not ifFalse:[1] ifTrue:[2] ' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

     m1 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a isNil ifTrue:[1] ifFalse:[2]' selector: #retFoo.
     m2 := RBMethod for:(RBClass named:'foo') source:'retFoo:a ^ a notNil not ifTrue:[1] ifFalse:[2]' selector: #retFoo.
     self assert:( RBCodeDuplicationRule hasMethod:m1 sameSemanticAs:m2).

    "Created: / 06-02-2017 / 12:05:16 / cg"
    "Modified: / 06-02-2017 / 16:10:34 / cg"
!

test18_relopChecks
     |node1 node2 mapping|

     mapping := Dictionary new.
     node1 := RBParser parseExpression:'a > 0'.
     node2 := RBParser parseExpression:'a > 0'.
     self assert:( node1 semanticallyEqualTo:node2 withMapping:mapping).

     node1 := RBParser parseExpression:'a > 0'.
     node2 := RBParser parseExpression:'0 < a'.
     self assert:( node1 semanticallyEqualTo:node2 withMapping:mapping).

     mapping := Dictionary new.
     node1 := RBParser parseExpression:'a > 0'.
     node2 := RBParser parseExpression:'b > 0'.
     self assert:( node1 semanticallyEqualTo:node2 withMapping:mapping).
     self assert:( mapping at:'a' ) = 'b'.

     mapping := Dictionary new.
     mapping at:'a' put:'a'.
     node1 := RBParser parseExpression:'a > 0'.
     node2 := RBParser parseExpression:'b > 0'.
     self deny:( node1 semanticallyEqualTo:node2 withMapping:mapping).
     "/ mapping should not be changed then
     self assert:( mapping at:'a' ) = 'a'.

     mapping := Dictionary new.
     mapping at:'a' put:'a'.
     node1 := RBParser parseExpression:'a > 0'.
     node2 := RBParser parseExpression:'(b <= 0) not'.
     self deny:( node1 semanticallyEqualTo:node2 withMapping:mapping).
     "/ mapping should not be changed then
     self assert:( mapping at:'a' ) = 'a'.

     mapping := Dictionary new.
     mapping at:'a' put:'a'.
     node1 := RBParser parseExpression:'a > 0'.
     node2 := RBParser parseExpression:'(b <= 0) not'.
     self deny:( node1 semanticallyEqualTo:node2 withMapping:mapping).
     "/ mapping should not be changed then
     self assert:( mapping at:'a' ) = 'a'.

    "Created: / 06-02-2017 / 12:33:45 / cg"
!

test19_relopChecks
     |node1 node2 mapping|

     mapping := Dictionary new.
     node1 := RBParser parseExpression:'a > 0'.
     node2 := RBParser parseExpression:'a <= 0'.
     self assert:( node1 semanticallyEqualToNegated:node2 withMapping:mapping).

     node1 := RBParser parseExpression:'a > 0'.
     node2 := RBParser parseExpression:'(a > 0) not'.
     self assert:( node1 semanticallyEqualToNegated:node2 withMapping:mapping).

    "Created: / 06-02-2017 / 13:38:18 / cg"
! !

!CodeDuplicationTest class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
! !

